#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022 Western Digital Corporation
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: wdc_redfish_command
short_description: Manages WDC UltraStar Data102 Out-Of-Band controllers using Redfish APIs
version_added: 5.4.0
description:
  - Builds Redfish URIs locally and sends them to remote OOB controllers to
    perform an action.
  - Manages OOB controller firmware. For example, Firmware Activate, Update and Activate.
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - List of commands to execute on OOB controller.
    type: list
    elements: str
  baseuri:
    description:
      - Base URI of OOB controller.  Must include this or I(ioms).
    type: str
  ioms:
    description:
      - List of IOM FQDNs for the enclosure.  Must include this or I(baseuri).
    type: list
    elements: str
  username:
    description:
      - User for authentication with OOB controller.
    type: str
  password:
    description:
      - Password for authentication with OOB controller.
    type: str
  auth_token:
    description:
      - Security token for authentication with OOB controller.
    type: str
  timeout:
    description:
      - Timeout in seconds for URL requests to OOB controller.
    default: 10
    type: int
  update_image_uri:
    required: false
    description:
      - The URI of the image for the update.
    type: str
  update_creds:
    required: false
    description:
      - The credentials for retrieving the update image.
    type: dict
    suboptions:
      username:
        required: false
        description:
          - The username for retrieving the update image.
        type: str
      password:
        required: false
        description:
          - The password for retrieving the update image.
        type: str
requirements:
  - dnspython (2.1.0 for Python 3, 1.16.0 for Python 2)
notes:
  - In the inventory, you can specify baseuri or ioms.  See the EXAMPLES section.
  - ioms is a list of FQDNs for the enclosure's IOMs.


author: Mike Moerk (@mikemoerk)
'''

EXAMPLES = '''
- name: Firmware Activate (required after SimpleUpdate to apply the new firmware)
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    ioms: "{{ ioms }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Firmware Activate with individual IOMs specified
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    ioms:
      - iom1.wdc.com
      - iom2.wdc.com
    username: "{{ username }}"
    password: "{{ password }}"

- name: Firmware Activate with baseuri specified
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    baseuri: "iom1.wdc.com"
    username: "{{ username }}"
    password: "{{ password }}"


- name: Update and Activate (orchestrates firmware update and activation with a single command)
  community.general.wdc_redfish_command:
    category: Update
    command: UpdateAndActivate
    ioms: "{{ ioms }}"
    username: "{{ username }}"
    password: "{{ password }}"
    update_image_uri: "{{ update_image_uri }}"
    update_creds:
      username: operator
      password: supersecretpwd
'''

RETURN = '''
msg:
    description: Message with action result or error description
    returned: always
    type: str
    sample: "Action was successful"
'''

from ansible_collections.community.general.plugins.module_utils.wdc_redfish_utils import WdcRedfishUtils
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

CATEGORY_COMMANDS_ALL = {
    "Update": [
        "FWActivate",
        "UpdateAndActivate"
    ]
}


def main():
    module = AnsibleModule(
        argument_spec=dict(
            category=dict(required=True),
            command=dict(required=True, type='list', elements='str'),
            ioms=dict(type='list', elements='str'),
            baseuri=dict(),
            username=dict(),
            password=dict(no_log=True),
            auth_token=dict(no_log=True),
            update_creds=dict(
                type='dict',
                options=dict(
                    username=dict(),
                    password=dict(no_log=True)
                )
            ),
            update_image_uri=dict(),
            timeout=dict(type='int', default=10)
        ),
        required_together=[
            ('username', 'password'),
        ],
        required_one_of=[
            ('username', 'auth_token'),
            ('baseuri', 'ioms')
        ],
        mutually_exclusive=[
            ('username', 'auth_token'),
        ],
        supports_check_mode=True
    )

    category = module.params['category']
    command_list = module.params['command']

    # admin credentials used for authentication
    creds = {'user': module.params['username'],
             'pswd': module.params['password'],
             'token': module.params['auth_token']}

    # timeout
    timeout = module.params['timeout']

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=to_native("Invalid Category '%s'. Valid Categories = %s" % (category, sorted(CATEGORY_COMMANDS_ALL.keys()))))

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=to_native("Invalid Command '%s'. Valid Commands = %s" % (cmd, CATEGORY_COMMANDS_ALL[category])))

    # Build root URI(s)
    if module.params.get("baseuri") is not None:
        root_uris = ["https://" + module.params['baseuri']]
    else:
        root_uris = [
            "https://" + iom for iom in module.params['ioms']
        ]
    rf_utils = WdcRedfishUtils(creds, root_uris, timeout, module,
                               resource_id=None, data_modification=True)

    # Organize by Categories / Commands

    if category == "Update":
        # execute only if we find UpdateService resources
        resource = rf_utils._find_updateservice_resource()
        if resource['ret'] is False:
            module.fail_json(msg=resource['msg'])
        # update options
        update_opts = {
            'update_creds': module.params['update_creds']
        }
        for command in command_list:
            if command == "FWActivate":
                if module.check_mode:
                    result = {
                        'ret': True,
                        'changed': True,
                        'msg': 'FWActivate not performed in check mode.'
                    }
                else:
                    result = rf_utils.firmware_activate(update_opts)
            elif command == "UpdateAndActivate":
                update_opts["update_image_uri"] = module.params['update_image_uri']
                result = rf_utils.update_and_activate(update_opts)

        if result['ret'] is False:
            module.fail_json(msg=to_native(result['msg']))
        else:
            del result['ret']
            changed = result.get('changed', True)
            session = result.get('session', dict())
            module.exit_json(changed=changed,
                             session=session,
                             msg='Action was successful' if not module.check_mode else result.get(
                                 'msg', "No action performed in check mode."
                             ))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright: (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: ldap_entry
short_description: Add or remove LDAP entries.
description:
  - Add or remove LDAP entries. This module only asserts the existence or
    non-existence of an LDAP entry, not its attributes. To assert the
    attribute values of an entry, see M(community.general.ldap_attrs).
notes:
  - The default authentication settings will attempt to use a SASL EXTERNAL
    bind over a UNIX domain socket. This works well with the default Ubuntu
    install for example, which includes a cn=peercred,cn=external,cn=auth ACL
    rule allowing root to modify the server configuration. If you need to use
    a simple bind to access your server, pass the credentials in I(bind_dn)
    and I(bind_pw).
author:
  - Jiri Tyr (@jtyr)
requirements:
  - python-ldap
options:
  attributes:
    description:
      - If I(state=present), attributes necessary to create an entry. Existing
        entries are never modified. To assert specific attribute values on an
        existing entry, use M(community.general.ldap_attrs) module instead.
    type: dict
  objectClass:
    description:
      - If I(state=present), value or list of values to use when creating
        the entry. It can either be a string or an actual list of
        strings.
    type: list
    elements: str
  state:
    description:
      - The target state of the entry.
    choices: [present, absent]
    default: present
    type: str
  recursive:
    description:
      - If I(state=delete), a flag indicating whether a single entry or the
        whole branch must be deleted.
    type: bool
    default: false
    version_added: 4.6.0
extends_documentation_fragment:
- community.general.ldap.documentation

'''


EXAMPLES = """
- name: Make sure we have a parent entry for users
  community.general.ldap_entry:
    dn: ou=users,dc=example,dc=com
    objectClass: organizationalUnit

- name: Make sure we have an admin user
  community.general.ldap_entry:
    dn: cn=admin,dc=example,dc=com
    objectClass:
      - simpleSecurityObject
      - organizationalRole
    attributes:
      description: An LDAP administrator
      userPassword: "{SSHA}tabyipcHzhwESzRaGA7oQ/SDoBZQOGND"

- name: Get rid of an old entry
  community.general.ldap_entry:
    dn: ou=stuff,dc=example,dc=com
    state: absent
    server_uri: ldap://localhost/
    bind_dn: cn=admin,dc=example,dc=com
    bind_pw: password

#
# The same as in the previous example but with the authentication details
# stored in the ldap_auth variable:
#
# ldap_auth:
#   server_uri: ldap://localhost/
#   bind_dn: cn=admin,dc=example,dc=com
#   bind_pw: password
#
# In the example below, 'args' is a task keyword, passed at the same level as the module
- name: Get rid of an old entry
  community.general.ldap_entry:
    dn: ou=stuff,dc=example,dc=com
    state: absent
  args: "{{ ldap_auth }}"
"""


RETURN = """
# Default return values
"""

import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_native, to_bytes
from ansible_collections.community.general.plugins.module_utils.ldap import LdapGeneric, gen_specs

LDAP_IMP_ERR = None
try:
    import ldap.modlist
    import ldap.controls

    HAS_LDAP = True
except ImportError:
    LDAP_IMP_ERR = traceback.format_exc()
    HAS_LDAP = False


class LdapEntry(LdapGeneric):
    def __init__(self, module):
        LdapGeneric.__init__(self, module)

        # Shortcuts
        self.state = self.module.params['state']
        self.recursive = self.module.params['recursive']

        # Add the objectClass into the list of attributes
        self.module.params['attributes']['objectClass'] = (
            self.module.params['objectClass'])

        # Load attributes
        if self.state == 'present':
            self.attrs = self._load_attrs()

    def _load_attrs(self):
        """ Turn attribute's value to array. """
        attrs = {}

        for name, value in self.module.params['attributes'].items():
            if isinstance(value, list):
                attrs[name] = list(map(to_bytes, value))
            else:
                attrs[name] = [to_bytes(value)]

        return attrs

    def add(self):
        """ If self.dn does not exist, returns a callable that will add it. """
        def _add():
            self.connection.add_s(self.dn, modlist)

        if not self._is_entry_present():
            modlist = ldap.modlist.addModlist(self.attrs)
            action = _add
        else:
            action = None

        return action

    def delete(self):
        """ If self.dn exists, returns a callable that will delete either
        the item itself if the recursive option is not set or the whole branch
        if it is. """
        def _delete():
            self.connection.delete_s(self.dn)

        def _delete_recursive():
            """ Attempt recurive deletion using the subtree-delete control.
            If that fails, do it manually. """
            try:
                subtree_delete = ldap.controls.ValueLessRequestControl('1.2.840.113556.1.4.805')
                self.connection.delete_ext_s(self.dn, serverctrls=[subtree_delete])
            except ldap.NOT_ALLOWED_ON_NONLEAF:
                search = self.connection.search_s(self.dn, ldap.SCOPE_SUBTREE, attrlist=('dn',))
                search.reverse()
                for entry in search:
                    self.connection.delete_s(entry[0])

        if self._is_entry_present():
            if self.recursive:
                action = _delete_recursive
            else:
                action = _delete
        else:
            action = None

        return action

    def _is_entry_present(self):
        try:
            self.connection.search_s(self.dn, ldap.SCOPE_BASE)
        except ldap.NO_SUCH_OBJECT:
            is_present = False
        else:
            is_present = True

        return is_present


def main():
    module = AnsibleModule(
        argument_spec=gen_specs(
            attributes=dict(default={}, type='dict'),
            objectClass=dict(type='list', elements='str'),
            state=dict(default='present', choices=['present', 'absent']),
            recursive=dict(default=False, type='bool'),
        ),
        required_if=[('state', 'present', ['objectClass'])],
        supports_check_mode=True,
    )

    if not HAS_LDAP:
        module.fail_json(msg=missing_required_lib('python-ldap'),
                         exception=LDAP_IMP_ERR)

    state = module.params['state']

    # Instantiate the LdapEntry object
    ldap = LdapEntry(module)

    # Get the action function
    if state == 'present':
        action = ldap.add()
    elif state == 'absent':
        action = ldap.delete()

    # Perform the action
    if action is not None and not module.check_mode:
        try:
            action()
        except Exception as e:
            module.fail_json(msg="Entry action failed.", details=to_native(e), exception=traceback.format_exc())

    module.exit_json(changed=(action is not None))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (C) 2020, Gonéri Le Bouder <goneri@lebouder.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: turbo_demo
short_description: A demo module for ansible_module.turbo
version_added: "1.0.0"
description:
- "This module is an example of an ansible_module.turbo integration."
author:
- Gonéri Le Bouder (@goneri)
"""

EXAMPLES = r"""
- name: Run the module
  cloud.common.turbo_demo:
"""

import os

from ansible_collections.cloud.common.plugins.module_utils.turbo.module import (
    AnsibleTurboModule as AnsibleModule,
)


def counter():
    counter.i += 1
    return counter.i


# NOTE: workaround to avoid a warning with ansible-doc
if True:  # pylint: disable=using-constant-test
    counter.i = 0


def get_message():
    return f"This is me running with PID: {os.getpid()}, called {counter.i} time(s)"


def run_module():
    result = {}

    # the AnsibleModule object will be our abstraction working with Ansible
    # this includes instantiation, a couple of common attr would be the
    # args/params passed to the execution, as well as if the module
    # supports check mode
    module = AnsibleModule(argument_spec={}, supports_check_mode=True)
    module.collection_name = "cloud.common"
    previous_value = counter.i
    if not module.check_mode:
        counter()
        result["changed"] = True
    result["message"] = get_message()
    result["counter"] = counter.i
    result["envvar"] = os.environ.get("TURBO_TEST_VAR")

    if module._diff:
        result["diff"] = {"before": previous_value, "after": counter.i}

    module.exit_json(**result)


def main():
    from ansible_collections.cloud.common.plugins.module_utils import turbo_demo

    run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pnp_device_unclaim
short_description: Resource module for Pnp Device Unclaim
description:
- Manage operation create of the resource Pnp Device Unclaim.
- Un-Claims one of more devices with specified workflow.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceIdList:
    description: Pnp Device Unclaim's deviceIdList.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.5.0
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) UnClaimDevice
  description: Complete reference of the UnClaimDevice API.
  link: https://developer.cisco.com/docs/dna-center/#!un-claim-device
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.un_claim_device,

  - Paths used are
    post /dna/intent/api/v1/onboarding/pnp-device/unclaim,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.pnp_device_unclaim:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceIdList:
    - string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "jsonArrayResponse": [
        {}
      ],
      "jsonResponse": {},
      "message": "string",
      "statusCode": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: wait_for_txt
short_description: Wait for TXT entries to be available on all authoritative nameservers
version_added: 0.1.0
description:
    - Wait for TXT entries with specific values to show up on B(all) authoritative nameservers for the DNS name.
author:
    - Felix Fontein (@felixfontein)
options:
    records:
        description:
            - A list of DNS names with TXT entries to look out for.
        required: true
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - A DNS name, like C(www.example.com).
                type: str
                required: true
            values:
                description:
                    - The TXT values to look for.
                type: list
                elements: str
                required: true
            mode:
                description:
                    - Comparison modes for the values in I(values).
                    - If C(subset), I(values) should be a (not necessarily proper) subset of the TXT values set for
                      the DNS name.
                    - If C(superset), I(values) should be a (not necessarily proper) superset of the TXT values set
                      for the DNS name.
                      This includes the case that no TXT entries are set.
                    - If C(superset_not_empty), I(values) should be a (not necessarily proper) superset of the TXT
                      values set for the DNS name, assuming at least one TXT record is present.
                    - If C(equals), I(values) should be the same set of strings as the TXT values for the DNS name
                      (up to order).
                    - If C(equals_ordered), I(values) should be the same ordered list of strings as the TXT values
                      for the DNS name.
                type: str
                default: subset
                choices:
                    - subset
                    - superset
                    - superset_not_empty
                    - equals
                    - equals_ordered
    query_retry:
        description:
            - Number of retries for DNS query timeouts.
        type: int
        default: 3
    query_timeout:
        description:
            - Timeout per DNS query in seconds.
        type: float
        default: 10
    timeout:
        description:
            - Global timeout for waiting for all records in seconds.
            - If not set, will wait indefinitely.
        type: float
    max_sleep:
        description:
            - Maximal amount of seconds to sleep between two rounds of probing the TXT records.
        type: float
        default: 10
    always_ask_default_resolver:
        description:
            - When set to C(true) (default), will use the default resolver to find the authoritative nameservers
              of a subzone.
            - When set to C(false), will use the authoritative nameservers of the parent zone to find the
              authoritative nameservers of a subzone. This only makes sense when the nameservers were recently
              changed and haven't propagated.
        type: bool
        default: true
requirements:
    - dnspython >= 1.15.0 (maybe older versions also work)
'''

EXAMPLES = r'''
- name: Wait for a TXT entry to appear
  community.dns.wait_for_txt:
    records:
      # We want that www.example.com has a single TXT record with value 'Hello world!'.
      # There should not be any other TXT record for www.example.com.
      - name: www.example.com
        values: "Hello world!"
        mode: equals
      # We want that example.com has a specific SPF record set.
      # We do not care about other TXT records.
      - name: www.example.com
        values: "v=spf1 a mx -all"
        mode: subset
'''

RETURN = r'''
records:
    description:
        - Results on the TXT records queried.
        - The entries are in a 1:1 correspondence to the entries of the I(records) parameter,
          in exactly the same order.
    returned: always
    type: list
    elements: dict
    contains:
        name:
            description:
                - The DNS name this check is for.
            returned: always
            type: str
            sample: example.com
        done:
            description:
                - Whether the check completed.
            returned: always
            type: bool
            sample: false
        values:
            description:
                - For every authoritative nameserver for the DNS name, lists the TXT records retrieved during the last lookup made.
                - Once the check completed for all TXT records retrieved, the TXT records for this DNS name are no longer checked.
                - If these are multiple TXT entries for a nameserver, the order is as it was received from that nameserver. This
                  might not be the same order provided in the check.
            returned: lookup was done at least once
            type: dict
            elements: list
            sample:
                ns1.example.com:
                    - TXT value 1
                    - TXT value 2
                ns2.example.com:
                    - TXT value 2
        check_count:
            description:
                - How often the TXT records for this DNS name were checked.
            returned: always
            type: int
            sample: 3
    sample:
        - name: example.com
          done: true
          values: [a, b, c]
          check_count: 1
        - name: foo.example.org
          done: false
          check_count: 0
completed:
    description:
        - How many of the checks were completed.
    returned: always
    type: int
    sample: 3
'''

import time
import traceback

try:
    from time import monotonic
except ImportError:
    from time import clock as monotonic

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native, to_text

from ansible_collections.community.dns.plugins.module_utils.resolver import (
    ResolveDirectlyFromNameServers,
    ResolverError,
    assert_requirements_present,
)

try:
    import dns.exception
    import dns.rdatatype
except ImportError:
    pass  # handled in assert_requirements_present()


def lookup(resolver, name):
    result = {}
    txts = resolver.resolve(name, rdtype=dns.rdatatype.TXT)
    for key, txt in txts.items():
        res = []
        if txt is not None:
            for data in txt:
                line = []
                for str in data.strings:
                    line.append(to_text(str))
                res.append(u''.join(line))
        result[key] = res
        txts[key] = []
    return result


def validate_check(record_values, expected_values, comparison_mode):
    if comparison_mode == 'subset':
        return set(expected_values) <= set(record_values)

    if comparison_mode == 'superset':
        return set(expected_values) >= set(record_values)

    if comparison_mode == 'superset_not_empty':
        return bool(record_values) and set(expected_values) >= set(record_values)

    if comparison_mode == 'equals':
        return sorted(record_values) == sorted(expected_values)

    if comparison_mode == 'equals_ordered':
        return record_values == expected_values

    raise Exception('Internal error!')


def main():
    module = AnsibleModule(
        argument_spec=dict(
            records=dict(required=True, type='list', elements='dict', options=dict(
                name=dict(required=True, type='str'),
                values=dict(required=True, type='list', elements='str'),
                mode=dict(type='str', default='subset', choices=['subset', 'superset', 'superset_not_empty', 'equals', 'equals_ordered']),
            )),
            query_retry=dict(type='int', default=3),
            query_timeout=dict(type='float', default=10),
            timeout=dict(type='float'),
            max_sleep=dict(type='float', default=10),
            always_ask_default_resolver=dict(type='bool', default=True),
        ),
    )
    assert_requirements_present(module)

    resolver = ResolveDirectlyFromNameServers(
        timeout=module.params['query_timeout'],
        timeout_retries=module.params['query_retry'],
        always_ask_default_resolver=module.params['always_ask_default_resolver'],
    )
    records = module.params['records']
    timeout = module.params['timeout']
    max_sleep = module.params['max_sleep']

    results = [None] * len(records)
    for index in range(len(records)):
        results[index] = {
            'name': records[index]['name'],
            'done': False,
            'check_count': 0,
        }
    finished_checks = 0

    start_time = monotonic()
    try:
        step = 0
        while True:
            has_timeout = False
            if timeout is not None:
                expired = monotonic() - start_time
                has_timeout = expired > timeout

            done = True
            for index, record in enumerate(records):
                if results[index]['done']:
                    continue
                txts = lookup(resolver, record['name'])
                results[index]['values'] = txts
                results[index]['check_count'] += 1
                if txts and all(validate_check(txt, record['values'], record['mode']) for txt in txts.values()):
                    results[index]['done'] = True
                    finished_checks += 1
                else:
                    done = False

            if done:
                module.exit_json(
                    msg='All checks passed',
                    records=results,
                    completed=finished_checks)

            if has_timeout:
                module.fail_json(
                    msg='Timeout ({0} out of {1} check(s) passed).'.format(finished_checks, len(records)),
                    records=results,
                    completed=finished_checks)

            # Simple quadratic sleep with maximum wait of max_sleep seconds
            wait = min(2 + step * 0.5, max_sleep)
            if timeout is not None:
                # Make sure we do not exceed the timeout by much by waiting
                expired = monotonic() - start_time
                wait = max(min(wait, timeout - expired + 0.1), 0.1)

            time.sleep(wait)
            step += 1
    except ResolverError as e:
        module.fail_json(
            msg='Unexpected resolving error: {0}'.format(to_native(e)),
            records=results,
            completed=finished_checks,
            exception=traceback.format_exc())
    except dns.exception.DNSException as e:
        module.fail_json(
            msg='Unexpected DNS error: {0}'.format(to_native(e)),
            records=results,
            completed=finished_checks,
            exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: iam_user
version_added: 1.0.0
short_description: Manage AWS IAM users
description:
  - A module to manage AWS IAM users.
  - The module does not manage groups that users belong to, groups memberships can be managed using M(community.aws.iam_group).
author: Josh Souza (@joshsouza)
options:
  name:
    description:
      - The name of the user to create.
    required: true
    type: str
  password:
    description:
      - The password to apply to the user.
    required: false
    type: str
    version_added: 2.2.0
  password_reset_required:
    description:
      - Defines if the user is required to set a new password after login.
    required: false
    type: bool
    default: false
    version_added: 3.1.0
  update_password:
    default: always
    choices: ['always', 'on_create']
    description:
      - When to update user passwords.
      - I(update_password=always) will ensure the password is set to I(password).
      - I(update_password=on_create) will only set the password for newly created users.
    type: str
    version_added: 2.2.0
  remove_password:
    description:
      - Option to delete user login passwords.
      - This field is mutually exclusive to I(password).
    type: 'bool'
    version_added: 2.2.0
  managed_policies:
    description:
      - A list of managed policy ARNs or friendly names to attach to the user.
      - To embed an inline policy, use M(community.aws.iam_policy).
    required: false
    type: list
    elements: str
    aliases: ['managed_policy']
  state:
    description:
      - Create or remove the IAM user.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  purge_policies:
    description:
      - When I(purge_policies=true) any managed policies not listed in I(managed_policies) will be detached.
    required: false
    default: false
    type: bool
    aliases: ['purge_policy', 'purge_managed_policies']
  tags:
    description:
      - Tag dict to apply to the user.
    required: false
    type: dict
    version_added: 2.1.0
  purge_tags:
    description:
      - Remove tags not listed in I(tags) when tags is specified.
    default: true
    type: bool
    version_added: 2.1.0
  wait:
    description:
      - When I(wait=True) the module will wait for up to I(wait_timeout) seconds
        for IAM user creation before returning.
    default: True
    type: bool
    version_added: 2.2.0
  wait_timeout:
    description:
      - How long (in seconds) to wait for creation / updates to complete.
    default: 120
    type: int
    version_added: 2.2.0
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = r'''
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Note: This module does not allow management of groups that users belong to.
#       Groups should manage their membership directly using community.aws.iam_group,
#       as users belong to them.

- name: Create a user
  community.aws.iam_user:
    name: testuser1
    state: present

- name: Create a user with a password
  community.aws.iam_user:
    name: testuser1
    password: SomeSecurePassword
    state: present

- name: Create a user and attach a managed policy using its ARN
  community.aws.iam_user:
    name: testuser1
    managed_policies:
      - arn:aws:iam::aws:policy/AmazonSNSFullAccess
    state: present

- name: Remove all managed policies from an existing user with an empty list
  community.aws.iam_user:
    name: testuser1
    state: present
    purge_policies: true

- name: Create user with tags
  community.aws.iam_user:
    name: testuser1
    state: present
    tags:
      Env: Prod

- name: Delete the user
  community.aws.iam_user:
    name: testuser1
    state: absent

'''
RETURN = r'''
user:
    description: dictionary containing all the user information
    returned: success
    type: complex
    contains:
        arn:
            description: the Amazon Resource Name (ARN) specifying the user
            type: str
            sample: "arn:aws:iam::1234567890:user/testuser1"
        create_date:
            description: the date and time, in ISO 8601 date-time format, when the user was created
            type: str
            sample: "2017-02-08T04:36:28+00:00"
        user_id:
            description: the stable and unique string identifying the user
            type: str
            sample: "AGPAIDBWE12NSFINE55TM"
        user_name:
            description: the friendly name that identifies the user
            type: str
            sample: "testuser1"
        path:
            description: the path to the user
            type: str
            sample: "/"
        tags:
            description: user tags
            type: dict
            returned: always
            sample: {"Env": "Prod"}
'''

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.core import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import compare_aws_tags


def compare_attached_policies(current_attached_policies, new_attached_policies):

    # If new_attached_policies is None it means we want to remove all policies
    if len(current_attached_policies) > 0 and new_attached_policies is None:
        return False

    current_attached_policies_arn_list = []
    for policy in current_attached_policies:
        current_attached_policies_arn_list.append(policy['PolicyArn'])

    if not set(current_attached_policies_arn_list).symmetric_difference(set(new_attached_policies)):
        return True
    else:
        return False


def convert_friendly_names_to_arns(connection, module, policy_names):

    # List comprehension that looks for any policy in the 'policy_names' list
    # that does not begin with 'arn'. If there aren't any, short circuit.
    # If there are, translate friendly name to the full arn
    if not any(not policy.startswith('arn:') for policy in policy_names if policy is not None):
        return policy_names
    allpolicies = {}
    paginator = connection.get_paginator('list_policies')
    policies = paginator.paginate().build_full_result()['Policies']

    for policy in policies:
        allpolicies[policy['PolicyName']] = policy['Arn']
        allpolicies[policy['Arn']] = policy['Arn']
    try:
        return [allpolicies[policy] for policy in policy_names]
    except KeyError as e:
        module.fail_json(msg="Couldn't find policy: " + str(e))


def wait_iam_exists(connection, module):

    user_name = module.params.get('name')
    wait_timeout = module.params.get('wait_timeout')

    delay = min(wait_timeout, 5)
    max_attempts = wait_timeout // delay

    try:
        waiter = connection.get_waiter('user_exists')
        waiter.wait(
            WaiterConfig={'Delay': delay, 'MaxAttempts': max_attempts},
            UserName=user_name,
        )
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg='Timeout while waiting on IAM user creation')
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg='Failed while waiting on IAM user creation')


def create_or_update_login_profile(connection, module):

    # Apply new password / update password for the user
    user_params = dict()
    user_params['UserName'] = module.params.get('name')
    user_params['Password'] = module.params.get('password')
    user_params['PasswordResetRequired'] = module.params.get('password_reset_required')
    retval = {}

    try:
        retval = connection.update_login_profile(**user_params)
    except is_boto3_error_code('NoSuchEntity'):
        # Login profile does not yet exist - create it
        try:
            retval = connection.create_login_profile(**user_params)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Unable to create user login profile")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unable to update user login profile")

    return True, retval


def delete_login_profile(connection, module):
    '''
    Deletes a users login profile.
        Parameters:
            connection: IAM client
            module: AWSModule
        Returns:
            (bool): True if login profile deleted, False if no login profile found to delete
    '''
    user_params = dict()
    user_params['UserName'] = module.params.get('name')

    # User does not have login profile - nothing to delete
    if not user_has_login_profile(connection, module, user_params['UserName']):
        return False

    if not module.check_mode:
        try:
            connection.delete_login_profile(**user_params)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:  # pylint: disable=duplicate-except
            module.fail_json_aws(e, msg="Unable to delete user login profile")

    return True


def create_or_update_user(connection, module):

    params = dict()
    params['UserName'] = module.params.get('name')
    managed_policies = module.params.get('managed_policies')
    purge_policies = module.params.get('purge_policies')

    if module.params.get('tags') is not None:
        params["Tags"] = ansible_dict_to_boto3_tag_list(module.params.get('tags'))

    changed = False

    if managed_policies:
        managed_policies = convert_friendly_names_to_arns(connection, module, managed_policies)

    # Get user
    user = get_user(connection, module, params['UserName'])

    # If user is None, create it
    new_login_profile = False
    if user is None:
        # Check mode means we would create the user
        if module.check_mode:
            module.exit_json(changed=True)

        try:
            connection.create_user(**params)
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Unable to create user")

        # Wait for user to be fully available before continuing
        if module.params.get('wait'):
            wait_iam_exists(connection, module)

        if module.params.get('password') is not None:
            login_profile_result, login_profile_data = create_or_update_login_profile(connection, module)

            if login_profile_data.get('LoginProfile', {}).get('PasswordResetRequired', False):
                new_login_profile = True
    else:
        login_profile_result = None
        update_result = update_user_tags(connection, module, params, user)

        if module.params['update_password'] == "always" and module.params.get('password') is not None:
            # Can't compare passwords, so just return changed on check mode runs
            if module.check_mode:
                module.exit_json(changed=True)
            login_profile_result, login_profile_data = create_or_update_login_profile(connection, module)

            if login_profile_data.get('LoginProfile', {}).get('PasswordResetRequired', False):
                new_login_profile = True

        elif module.params.get('remove_password'):
            login_profile_result = delete_login_profile(connection, module)

        changed = bool(update_result) or bool(login_profile_result)

    # Manage managed policies
    current_attached_policies = get_attached_policy_list(connection, module, params['UserName'])
    if not compare_attached_policies(current_attached_policies, managed_policies):
        current_attached_policies_arn_list = []
        for policy in current_attached_policies:
            current_attached_policies_arn_list.append(policy['PolicyArn'])

        # If managed_policies has a single empty element we want to remove all attached policies
        if purge_policies:
            # Detach policies not present
            for policy_arn in list(set(current_attached_policies_arn_list) - set(managed_policies)):
                changed = True
                if not module.check_mode:
                    try:
                        connection.detach_user_policy(UserName=params['UserName'], PolicyArn=policy_arn)
                    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                        module.fail_json_aws(e, msg="Unable to detach policy {0} from user {1}".format(
                                             policy_arn, params['UserName']))

        # If there are policies to adjust that aren't in the current list, then things have changed
        # Otherwise the only changes were in purging above
        if set(managed_policies).difference(set(current_attached_policies_arn_list)):
            changed = True
            # If there are policies in managed_policies attach each policy
            if managed_policies != [None] and not module.check_mode:
                for policy_arn in managed_policies:
                    try:
                        connection.attach_user_policy(UserName=params['UserName'], PolicyArn=policy_arn)
                    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                        module.fail_json_aws(e, msg="Unable to attach policy {0} to user {1}".format(
                                             policy_arn, params['UserName']))

    if module.check_mode:
        module.exit_json(changed=changed)

    # Get the user again
    user = get_user(connection, module, params['UserName'])
    if changed and new_login_profile:
        # `LoginProfile` is only returned on `create_login_profile` method
        user['user']['password_reset_required'] = login_profile_data.get('LoginProfile', {}).get('PasswordResetRequired', False)

    module.exit_json(changed=changed, iam_user=user, user=user['user'])


def destroy_user(connection, module):

    user_name = module.params.get('name')

    user = get_user(connection, module, user_name)
    # User is not present
    if not user:
        module.exit_json(changed=False)

    # Check mode means we would remove this user
    if module.check_mode:
        module.exit_json(changed=True)

    # Remove any attached policies otherwise deletion fails
    try:
        for policy in get_attached_policy_list(connection, module, user_name):
            connection.detach_user_policy(UserName=user_name, PolicyArn=policy['PolicyArn'])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Unable to delete user {0}".format(user_name))

    try:
        # Remove user's access keys
        access_keys = connection.list_access_keys(UserName=user_name)["AccessKeyMetadata"]
        for access_key in access_keys:
            connection.delete_access_key(UserName=user_name, AccessKeyId=access_key["AccessKeyId"])

        # Remove user's login profile (console password)
        delete_login_profile(connection, module)

        # Remove user's ssh public keys
        ssh_public_keys = connection.list_ssh_public_keys(UserName=user_name)["SSHPublicKeys"]
        for ssh_public_key in ssh_public_keys:
            connection.delete_ssh_public_key(UserName=user_name, SSHPublicKeyId=ssh_public_key["SSHPublicKeyId"])

        # Remove user's service specific credentials
        service_credentials = connection.list_service_specific_credentials(UserName=user_name)["ServiceSpecificCredentials"]
        for service_specific_credential in service_credentials:
            connection.delete_service_specific_credential(
                UserName=user_name,
                ServiceSpecificCredentialId=service_specific_credential["ServiceSpecificCredentialId"]
            )

        # Remove user's signing certificates
        signing_certificates = connection.list_signing_certificates(UserName=user_name)["Certificates"]
        for signing_certificate in signing_certificates:
            connection.delete_signing_certificate(
                UserName=user_name,
                CertificateId=signing_certificate["CertificateId"]
            )

        # Remove user's MFA devices
        mfa_devices = connection.list_mfa_devices(UserName=user_name)["MFADevices"]
        for mfa_device in mfa_devices:
            connection.deactivate_mfa_device(UserName=user_name, SerialNumber=mfa_device["SerialNumber"])

        # Remove user's inline policies
        inline_policies = connection.list_user_policies(UserName=user_name)["PolicyNames"]
        for policy_name in inline_policies:
            connection.delete_user_policy(UserName=user_name, PolicyName=policy_name)

        # Remove user's group membership
        user_groups = connection.list_groups_for_user(UserName=user_name)["Groups"]
        for group in user_groups:
            connection.remove_user_from_group(UserName=user_name, GroupName=group["GroupName"])

        connection.delete_user(UserName=user_name)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Unable to delete user {0}".format(user_name))

    module.exit_json(changed=True)


def get_user(connection, module, name):

    params = dict()
    params['UserName'] = name

    try:
        user = connection.get_user(**params)
    except is_boto3_error_code('NoSuchEntity'):
        return None
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unable to get user {0}".format(name))

    tags = boto3_tag_list_to_ansible_dict(user['User'].pop('Tags', []))
    user = camel_dict_to_snake_dict(user)
    user['user']['tags'] = tags
    return user


def get_attached_policy_list(connection, module, name):

    try:
        return connection.list_attached_user_policies(UserName=name)['AttachedPolicies']
    except is_boto3_error_code('NoSuchEntity'):
        return None
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unable to get policies for user {0}".format(name))


def user_has_login_profile(connection, module, name):
    '''
    Returns whether or not given user has a login profile.
        Parameters:
            connection: IAM client
            module: AWSModule
            name (str): Username of user
        Returns:
            (bool): True if user had login profile, False if not
    '''
    try:
        connection.get_login_profile(UserName=name)
    except is_boto3_error_code('NoSuchEntity'):
        return False
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unable to get login profile for user {0}".format(name))
    return True


def update_user_tags(connection, module, params, user):
    user_name = params['UserName']
    existing_tags = user['user']['tags']
    new_tags = params.get('Tags')
    if new_tags is None:
        return False
    new_tags = boto3_tag_list_to_ansible_dict(new_tags)

    purge_tags = module.params.get('purge_tags')

    tags_to_add, tags_to_remove = compare_aws_tags(existing_tags, new_tags, purge_tags=purge_tags)

    if not module.check_mode:
        try:
            if tags_to_remove:
                connection.untag_user(UserName=user_name, TagKeys=tags_to_remove)
            if tags_to_add:
                connection.tag_user(UserName=user_name, Tags=ansible_dict_to_boto3_tag_list(tags_to_add))
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg='Unable to set tags for user %s' % user_name)

    changed = bool(tags_to_add) or bool(tags_to_remove)
    return changed


def main():

    argument_spec = dict(
        name=dict(required=True, type='str'),
        password=dict(type='str', no_log=True),
        password_reset_required=dict(type='bool', default=False, no_log=False),
        update_password=dict(default='always', choices=['always', 'on_create'], no_log=False),
        remove_password=dict(type='bool', no_log=False),
        managed_policies=dict(default=[], type='list', aliases=['managed_policy'], elements='str'),
        state=dict(choices=['present', 'absent'], required=True),
        purge_policies=dict(default=False, type='bool', aliases=['purge_policy', 'purge_managed_policies']),
        tags=dict(type='dict'),
        purge_tags=dict(type='bool', default=True),
        wait=dict(type='bool', default=True),
        wait_timeout=dict(default=120, type='int'),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[['password', 'remove_password']],
    )

    module.deprecate("The 'iam_user' return key is deprecated and will be replaced by 'user'. Both values are returned for now.",
                     date='2024-05-01', collection_name='community.aws')

    connection = module.client('iam')

    state = module.params.get("state")

    if state == 'present':
        create_or_update_user(connection, module)
    else:
        destroy_user(connection, module)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.internal_test_tools.tests.unit.utils.fetch_url_module_framework import (
    BaseTestModule,
    FetchUrlCall,
)

from ansible_collections.community.dns.plugins.modules import hetzner_dns_zone_info

# These imports are needed so patching below works
import ansible_collections.community.dns.plugins.module_utils.http  # noqa

from .hetzner import (
    HETZNER_JSON_ZONE_GET_RESULT,
    HETZNER_JSON_ZONE_LIST_RESULT,
)


class TestHetznerDNSZoneInfoJSON(BaseTestModule):
    MOCK_ANSIBLE_MODULEUTILS_BASIC_ANSIBLEMODULE = 'ansible_collections.community.dns.plugins.modules.hetzner_dns_zone_info.AnsibleModule'
    MOCK_ANSIBLE_MODULEUTILS_URLS_FETCH_URL = 'ansible_collections.community.dns.plugins.module_utils.http.fetch_url'

    def test_unknown_zone(self, mocker):
        result = self.run_module_failed(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones', without_query=True)
            .expect_query_values('name', 'example.org')
            .return_header('Content-Type', 'application/json; charset=utf-8')
            .result_json(HETZNER_JSON_ZONE_LIST_RESULT),
        ])

        assert result['msg'] == 'Zone not found'

    def test_unknown_zone_id(self, mocker):
        result = self.run_module_failed(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_id': '23',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 404)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones/23')
            .return_header('Content-Type', 'application/json; charset=utf-8')
            .result_json(dict(message="")),
        ])

        assert result['msg'] == 'Zone not found'

    def test_auth_error(self, mocker):
        result = self.run_module_failed(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 401)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones', without_query=True)
            .expect_query_values('name', 'example.org')
            .result_str(''),
        ])

        assert result['msg'] == 'Cannot authenticate: Unauthorized: the authentication parameters are incorrect (HTTP status 401)'

    def test_auth_error_forbidden(self, mocker):
        result = self.run_module_failed(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_id': '23',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 403)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones/23')
            .result_json(dict(message="")),
        ])

        assert result['msg'] == 'Cannot authenticate: Forbidden: you do not have access to this resource (HTTP status 403)'

    def test_other_error(self, mocker):
        result = self.run_module_failed(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 500)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones', without_query=True)
            .expect_query_values('name', 'example.org')
            .result_str(''),
        ])

        assert result['msg'].startswith('Error: GET https://dns.hetzner.com/api/v1/zones?')
        assert 'did not yield JSON data, but HTTP status code 500 with Content-Type' in result['msg']

    def test_get(self, mocker):
        result = self.run_module_success(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_name': 'example.com',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones', without_query=True)
            .expect_query_values('name', 'example.com')
            .return_header('Content-Type', 'application/json; charset=utf-8')
            .result_json(HETZNER_JSON_ZONE_LIST_RESULT),
        ])
        assert result['changed'] is False
        assert result['zone_id'] == '42'
        assert result['zone_name'] == 'example.com'
        assert result['zone_info'] == {
            'created': '2021-07-09T11:18:37Z',
            'modified': '2021-07-09T11:18:37Z',
            'legacy_dns_host': 'string',
            'legacy_ns': ['bar', 'foo'],
            'ns': ['string'],
            'owner': 'Example',
            'paused': True,
            'permission': 'string',
            'project': 'string',
            'registrar': 'string',
            'status': 'verified',
            'ttl': 10800,
            'verified': '2021-07-09T11:18:37Z',
            'records_count': 0,
            'is_secondary_dns': True,
            'txt_verification': {
                'name': 'string',
                'token': 'string',
            },
        }

    def test_get_id(self, mocker):
        result = self.run_module_success(mocker, hetzner_dns_zone_info, {
            'hetzner_token': 'foo',
            'zone_id': '42',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('auth-api-token', 'foo')
            .expect_url('https://dns.hetzner.com/api/v1/zones/42')
            .return_header('Content-Type', 'application/json; charset=utf-8')
            .result_json(HETZNER_JSON_ZONE_GET_RESULT),
        ])
        assert result['changed'] is False
        assert result['zone_id'] == '42'
        assert result['zone_name'] == 'example.com'
        assert result['zone_info'] == {
            'created': '2021-07-09T11:18:37Z',
            'modified': '2021-07-09T11:18:37Z',
            'legacy_dns_host': 'string',
            'legacy_ns': ['bar', 'foo'],
            'ns': ['string'],
            'owner': 'Example',
            'paused': True,
            'permission': 'string',
            'project': 'string',
            'registrar': 'string',
            'status': 'verified',
            'ttl': 10800,
            'verified': '2021-07-09T11:18:37Z',
            'records_count': 0,
            'is_secondary_dns': True,
            'txt_verification': {
                'name': 'string',
                'token': 'string',
            },
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type


import pytest

from ansible_collections.community.dns.plugins.module_utils.record import (
    DNSRecord,
)

from ansible_collections.community.dns.plugins.module_utils.zone import (
    DNSZone,
    DNSZoneWithRecords,
)


def test_zone_str_repr():
    Z1 = DNSZone('foo')
    assert str(Z1) == 'DNSZone(name: foo, info: {})'
    assert repr(Z1) == 'DNSZone(name: foo, info: {})'
    Z2 = DNSZone('foo')
    Z2.id = 42
    Z2.info['foo'] = 'bar'
    assert str(Z2) == "DNSZone(id: 42, name: foo, info: {'foo': 'bar'})"
    assert repr(Z2) == "DNSZone(id: 42, name: foo, info: {'foo': 'bar'})"


def test_zone_with_records_str_repr():
    Z1 = DNSZone('foo')
    Z2 = DNSZone('foo')
    Z2.id = 42
    A1 = DNSRecord()
    A1.prefix = None
    A1.type = 'A'
    A1.ttl = 300
    A1.target = '1.2.3.4'
    A2 = DNSRecord()
    A2.id = 23
    A2.prefix = 'bar'
    A2.type = 'A'
    A2.ttl = 1
    A2.target = ''
    A2.extra['foo'] = 23
    ZZ1 = DNSZoneWithRecords(Z1, [A1])
    ZZ2 = DNSZoneWithRecords(Z2, [A1, A2])
    assert str(ZZ1) == '(DNSZone(name: foo, info: {}), [DNSRecord(type: A, prefix: (none), target: "1.2.3.4", ttl: 5m)])'
    assert repr(ZZ1) == 'DNSZoneWithRecords(DNSZone(name: foo, info: {}), [DNSRecord(type: A, prefix: (none), target: "1.2.3.4", ttl: 5m)])'
    assert str(ZZ2) == (
        '(DNSZone(id: 42, name: foo, info: {}), [DNSRecord(type: A, prefix: (none), target: "1.2.3.4", ttl: 5m),'
        ' DNSRecord(id: 23, type: A, prefix: "bar", target: "", ttl: 1s, extra: {\'foo\': 23})])'
    )
    assert repr(ZZ2) == (
        'DNSZoneWithRecords(DNSZone(id: 42, name: foo, info: {}), [DNSRecord(type: A, prefix: (none), target: "1.2.3.4", ttl: 5m),'
        ' DNSRecord(id: 23, type: A, prefix: "bar", target: "", ttl: 1s, extra: {\'foo\': 23})])'
    )



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The acls parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""
import re
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network_template import (
    NetworkTemplate,
)


def _tmplt_access_list_entries(aces):
    def source_destination_common_config(config_data, command, attr):
        if config_data[attr].get("address"):
            command += " {address}".format(**config_data[attr])
            if config_data[attr].get("wildcard_bits"):
                command += " {wildcard_bits}".format(**config_data[attr])
        elif config_data[attr].get("any"):
            command += " any".format(**config_data[attr])
        elif config_data[attr].get("host"):
            command += " host {host}".format(**config_data[attr])
        elif config_data[attr].get("object_group"):
            command += " object-group {object_group}".format(
                **config_data[attr]
            )
        if config_data[attr].get("port_protocol"):
            if config_data[attr].get("port_protocol").get("range"):
                command += " range {0} {1}".format(
                    config_data[attr]["port_protocol"]["range"].get("start"),
                    config_data[attr]["port_protocol"]["range"].get("end"),
                )
            else:
                port_proto_type = list(
                    config_data[attr]["port_protocol"].keys()
                )[0]
                command += " {0} {1}".format(
                    port_proto_type,
                    config_data[attr]["port_protocol"][port_proto_type],
                )
        return command

    command = ""
    proto_option = None
    if aces:
        if aces.get("sequence") and aces.get("afi") == "ipv4":
            command += "{sequence}".format(**aces)
        if (
            aces.get("grant")
            and aces.get("sequence")
            and aces.get("afi") == "ipv4"
        ):
            command += " {grant}".format(**aces)
        elif (
            aces.get("grant")
            and aces.get("sequence")
            and aces.get("afi") == "ipv6"
        ):
            command += "{grant}".format(**aces)
        elif aces.get("grant"):
            command += "{grant}".format(**aces)
        if aces.get("protocol_options"):
            if "protocol_number" in aces["protocol_options"]:
                command += " {protocol_number}".format(
                    **aces["protocol_options"]
                )
            else:
                command += " {0}".format(list(aces["protocol_options"])[0])
                proto_option = aces["protocol_options"].get(
                    list(aces["protocol_options"])[0]
                )
        elif aces.get("protocol"):
            command += " {protocol}".format(**aces)
        if aces.get("source"):
            command = source_destination_common_config(aces, command, "source")
        if aces.get("destination"):
            command = source_destination_common_config(
                aces, command, "destination"
            )
        if isinstance(proto_option, dict):
            command += " {0}".format(
                list(proto_option.keys())[0].replace("_", "-")
            )
        if aces.get("dscp"):
            command += " dscp {dscp}".format(**aces)
        if aces.get("sequence") and aces.get("afi") == "ipv6":
            command += " sequence {sequence}".format(**aces)
        if aces.get("enable_fragments") or aces.get("fragments"):
            command += " fragments"
        if aces.get("log"):
            command += " log"
            if aces["log"].get("user_cookie"):
                command += " {user_cookie}".format(**aces["log"])
        if aces.get("log_input"):
            command += " log-input"
            if aces["log_input"].get("user_cookie"):
                command += " {user_cookie}".format(**aces["log_input"])
        if aces.get("option"):
            option_val = list(aces.get("option").keys())[0]
            command += " option {0}".format(option_val)
        if aces.get("precedence"):
            command += " precedence {precedence}".format(**aces)
        if aces.get("time_range"):
            command += " time-range {time_range}".format(**aces)
        if aces.get("tos"):
            command += " tos"
            if aces["tos"].get("service_value"):
                command += " {service_value}".format(**aces["tos"])
            elif aces["tos"].get("max_reliability"):
                command += " max-reliability"
            elif aces["tos"].get("max_throughput"):
                command += " max-throughput"
            elif aces["tos"].get("min_delay"):
                command += " min-delay"
            elif aces["tos"].get("min_monetary_cost"):
                command += " min-monetary-cost"
            elif aces["tos"].get("normal"):
                command += " normal"
        if aces.get("ttl"):
            command += " ttl {0}".format(list(aces["ttl"])[0])
            proto_option = aces["ttl"].get(list(aces["ttl"])[0])
            command += " {0}".format(proto_option)
    return command


class AclsTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(AclsTemplate, self).__init__(lines=lines, tmplt=self)

    PARSERS = [
        {
            "name": "acls_name",
            "getval": re.compile(
                r"""^(?P<acl_type>Standard|Extended|Reflexive)*
                    \s*(?P<afi>IP|IPv6)*
                    \s*access*
                    \s*list*
                    \s*(?P<acl_name>.+)*
                    $""",
                re.VERBOSE,
            ),
            "compval": "name",
            "setval": "name",
            "result": {
                "acls": {
                    "{{ acl_name|d() }}": {
                        "name": "{{ acl_name }}",
                        "acl_type": "{{ acl_type.lower() if acl_type is defined }}",
                        "afi": "{{ 'ipv4' if afi == 'IP' else 'ipv6' }}",
                    }
                }
            },
            "shared": True,
        },
        {
            "name": "_acls_name",
            "getval": re.compile(
                r"""^(ip|ipv6)
                    (\s(access-list))
                    (\s(standard|extended))
                    (\s(?P<acl_name_r>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "compval": "name",
            "setval": "ip access-list",
            "result": {},
            "shared": True,
        },
        {
            "name": "remarks",
            "getval": re.compile(
                r"""\s+remark
                    (\s(?P<remarks>.+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": "remark {{ remarks }}",
            "result": {
                "acls": {
                    "{{ acl_name_r|d() }}": {
                        "name": "{{ acl_name_r }}",
                        "aces": [{"remarks": "{{ remarks }}"}],
                    }
                }
            },
        },
        {
            "name": "remarks_type_linear",
            "getval": re.compile(
                r"""^(access-list)
                    (\s(?P<acl_name_linear>\S+))?
                    (\sremark\s(?P<remarks>.+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": "remark {{ remarks }}",
            "result": {
                "acls": {
                    "{{ acl_name_linear|d() }}": {
                        "name": "{{ acl_name_linear }}",
                        "aces": [{"remarks": "{{ remarks }}"}],
                    }
                }
            },
        },
        {
            "name": "aces_ipv4_standard",
            "getval": re.compile(
                r"""\s*(?P<sequence>\d+)*
                        \s(?P<grant>deny|permit)?
                        (\s+(?P<address>(?!ahp|eigrp|esp|gre|icmp|igmp|ipv6|ipinip|ip|nos|object-group|ospf|pcp|pim|sctp|tcp|udp)\S+|\S+,))?
                        (\s*(?P<any>any))?
                        (\swildcard\sbits\s(?P<wildcard>\S+))?
                        (\shost\s(?P<host>\S+))?
                        (\s(?P<log>log))?
                    $""",
                re.VERBOSE,
            ),
            "compval": "aces",
            "result": {
                "acls": {
                    "{{ acl_name|d() }}": {
                        "name": "{{ acl_name }}",
                        "aces": [
                            {
                                "sequence": "{{ sequence }}",
                                "grant": "{{ grant }}",
                                "source": {
                                    "address": "{{ address }}",
                                    "wildcard_bits": "{{ wildcard }}",
                                    "any": "{{ not not any }}",
                                    "host": "{{ host }}",
                                },
                                "log": {"set": "{{ not not log }}"},
                            }
                        ],
                    }
                }
            },
        },
        {
            "name": "aces",
            "getval": re.compile(
                r"""\s*(?P<sequence>\d+)*
                        \s*(?P<grant>deny|permit)*
                        (\sevaluate\s(?P<evaluate>\S+))?
                        (\s(?P<protocol>ahp|eigrp|esp|gre|icmp|igmp|ipv6|ipinip|ip|nos|object-group|ospf|pcp|pim|sctp|tcp|udp))?
                        (\s(?P<protocol_num>\d+))?
                        (\s(?P<source>(any|\S+\s\S+|host\s\S+|object-group\s\S+))?)
                        (\s(?P<source_port_protocol>(eq|gts|gt|lt|neq)\s(\S+|\d+)))?
                        (\srange\s(?P<srange_start>\d+)\s(?P<srange_end>\d+))?
                        (\s(?P<destination>(any|\S+\s\S+|host\s\S+|object-group\s\S+))?)
                        (\s(?P<dest_port_protocol>(eq|gts|lt|neq)\s(\S+|\d+)))?
                        (\srange\s(?P<drange_start>\d+)\s(?P<drange_end>\d+))?
                        (\s(?P<icmp_igmp_tcp_protocol>administratively-prohibited|alternate-address|conversion-error|dod-host-prohibited|dod-net-prohibited|echo|echo-reply|general-parameter-problem|host-isolated|host-precedence-unreachable|host-redirect|host-tos-redirect|host-tos-unreachable|host-unknown|host-unreachable|information-reply|information-request|mask-reply|mask-request|mobile-redirect|net-redirect|net-tos-redirect|net-tos-unreachable|net-unreachable|network-unknown|no-room-for-option|option-missing|packet-too-big|parameter-problem|port-unreachable|precedence-unreachable|protocol-unreachable|reassembly-timeout|redirect|router-advertisement|router-solicitation|source-quench|source-route-failed|time-exceeded|timestamp-reply|timestamp-request|traceroute|ttl-exceeded|unreachable|dvmrp|host-query|mtrace-resp|mtrace-route|pim|trace|v1host-report|v2host-report|v2leave-group|v3host-report|ack|established|fin|psh|rst|syn|urg))?
                        (\sdscp\s(?P<dscp>\S+))?
                        (\s(?P<enable_fragments>fragments))?
                        (\s(?P<log_input>log-input\s\(tag\s=\s\S+\)|log-input))?
                        (\s(?P<log>log\s\(tag\s=\s\S+\)|log))?
                        (\soption\s(?P<option>\S+|\d+))?
                        (\sprecedence\s(?P<precedence>\S+|\d+))?
                        (\stime-range\s(?P<time_range>\S+))?
                        (\stos\s(?P<tos>\S+|\d+))?
                        (\sttl\seq\s(?P<ttl_eq>\d+))?
                        (\sttl\sgt\s(?P<ttl_gt>\d+))?
                        (\sttl\slt\s(?P<ttl_lt>\d+))?
                        (\sttl\sneg\s(?P<ttl_neg>\d+))?
                        (\ssequence\s(?P<sequence_ipv6>\d+))?
                    """,
                re.VERBOSE,
            ),
            "setval": _tmplt_access_list_entries,
            "compval": "aces",
            "result": {
                "acls": {
                    "{{ acl_name|d() }}": {
                        "name": "{{ acl_name }}",
                        "aces": [
                            {
                                "sequence": "{% if sequence is defined %}{{ sequence \
                                    }}{% elif sequence_ipv6 is defined %}{{ sequence_ipv6 }}{% endif %}",
                                "grant": "{{ grant }}",
                                "evaluate": "{{ evaluate }}",
                                "protocol": "{{ protocol }}",
                                "protocol_number": "{{ protocol_num }}",
                                "icmp_igmp_tcp_protocol": "{{ icmp_igmp_tcp_protocol }}",
                                "source": {
                                    "address": "{% if source is defined and '.' in source and 'host' not in source %}{{\
                                        source.split(' ')[0] }}{% elif source is defined and '::' in source %}{{ source }}{% endif %}",
                                    "wildcard_bits": "{{ source.split(' ')[1] if source is defined and '.' in source and 'host' not in source }}",
                                    "any": "{{ True if source is defined and source == 'any' }}",
                                    "host": "{{ source.split(' ')[1] if source is defined and 'host' in source }}",
                                    "object_group": "{{ source.split(' ')[1] if source is defined and 'object-group' in source }}",
                                    "port_protocol": {
                                        "{{ source_port_protocol.split(' ')[0] if source_port_protocol is defined else None }}": "{{\
                                            source_port_protocol.split(' ')[1] if source_port_protocol is defined else None }}",
                                        "range": {
                                            "start": "{{ srange_start if srange_start is defined else None }}",
                                            "end": "{{ srange_end if srange_end is defined else None }}",
                                        },
                                    },
                                },
                                "destination": {
                                    "address": "{% if destination is defined and '.' in destination and 'host' not in destination %}{{\
                                        destination.split(' ')[0] }}{% elif std_dest is defined and '.' in std_dest and 'host' not in std_dest %}{{\
                                            std_dest.split(' ')[0] }}{% elif destination is defined and '::' in destination %}{{ destination }}{% endif %}",
                                    "wildcard_bits": "{% if destination is defined and '.' in destination and 'host' not in destination %}{{\
                                        destination.split(' ')[1] }}{% elif std_dest is defined and '.' in std_dest and 'host' not in std_dest %}{{\
                                            std_dest.split(' ')[1] }}{% endif %}",
                                    "any": "{{ True if destination is defined and destination == 'any' else None }}",
                                    "host": "{{ destination.split(' ')[1] if destination is defined and 'host' in destination }}",
                                    "object_group": "{{ destination.split(' ')[1] if destination is defined and 'object-group' in destination else None }}",
                                    "port_protocol": {
                                        "{{ dest_port_protocol.split(' ')[0] if dest_port_protocol is defined else None }}": "{{\
                                            dest_port_protocol.split(' ')[1] if dest_port_protocol is defined else None }}",
                                        "range": {
                                            "start": "{{ drange_start if drange_start is defined else None }}",
                                            "end": "{{ drange_end if drange_end is defined else None }}",
                                        },
                                    },
                                },
                                "dscp": "{{ dscp }}",
                                "enable_fragments": "{{ True if enable_fragments is defined else None }}",
                                "log": {
                                    "set": "{{ True if log is defined and 'tag' not in log else '' }}",
                                    "user_cookie": "{{ log.split(' ')[-1].split(')')[0] if log is defined and 'tag' in log else '' }}",
                                },
                                "log_input": {
                                    "set": "{{ True if log_input is defined and 'tag' not in log_input else '' }}",
                                    "user_cookie": "{{ log_input.split(' ')[-1].split(')')[0] if log_input is defined and 'tag' in log_input }}",
                                },
                                "option": {
                                    "{{ option if option is defined else None }}": "{{ True if option is defined else None }}"
                                },
                                "precedence": "{{ precedence }}",
                                "time_range": "{{ time_range }}",
                                "tos": {
                                    "max_reliability": "{{ True if tos is defined and 'max-reliability' in tos else '' }}",
                                    "max_throughput": "{{ True if tos is defined and 'max-throughput' in tos else '' }}",
                                    "min_delay": "{{ True if tos is defined and 'min-delay' in tos else '' }}",
                                    "min_monetary_cost": "{{ True if tos is defined and 'min-monetary-cost' in tos else '' }}",
                                    "normal": "{{ True if tos is defined and 'normal' in tos else '' }}",
                                    "service_value": "{{ tos if tos is defined else None }}",
                                },
                                "ttl": {
                                    "eq": "{{ ttl_eq }}",
                                    "gt": "{{ ttl_gt }}",
                                    "lt": "{{ ttl_lt }}",
                                    "neq": "{{ ttl_neq }}",
                                },
                            }
                        ],
                    }
                }
            },
        },
    ]



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.cisco.nxos.plugins.modules import nxos_evpn_vni
from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch

from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosEvpnVniModule(TestNxosModule):

    module = nxos_evpn_vni

    def setUp(self):
        super(TestNxosEvpnVniModule, self).setUp()

        self.mock_load_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_evpn_vni.load_config",
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_config = patch(
            "ansible_collections.cisco.nxos.plugins.modules.nxos_evpn_vni.get_config",
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosEvpnVniModule, self).tearDown()
        self.mock_load_config.stop()
        self.mock_get_config.stop()

    def load_fixtures(self, commands=None, device=""):
        self.get_config.return_value = load_fixture("", "nxos_evpn_vni_config.cfg")
        self.load_config.return_value = None

    def test_nxos_evpn_vni_present(self):
        set_module_args(dict(vni="6000", route_target_import="5000:10", state="present"))
        result = self.execute_module(changed=True)
        self.assertEqual(
            result["commands"],
            [
                "evpn",
                "vni 6000 l2",
                "route-target import 5000:10",
                "no route-target import auto",
            ],
        )

    def test_nxos_evpn_vni_absent_not_existing(self):
        set_module_args(dict(vni="12000", state="absent"))
        result = self.execute_module(changed=False)
        self.assertEqual(result["commands"], [])

    def test_nxos_evpn_vni_absent_existing(self):
        set_module_args(dict(vni="6000", state="absent"))
        result = self.execute_module(changed=True)
        self.assertEqual(result["commands"], ["evpn", "no vni 6000 l2"])



# (c) 2021 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from textwrap import dedent
from ansible_collections.cisco.iosxr.tests.unit.compat.mock import patch
from ansible_collections.cisco.iosxr.plugins.modules import (
    iosxr_logging_global,
)
from ansible_collections.cisco.iosxr.tests.unit.modules.utils import (
    set_module_args,
)
from .iosxr_module import TestIosxrModule


class TestIosxrLoggingGlobalModule(TestIosxrModule):
    module = iosxr_logging_global

    def setUp(self):
        super(TestIosxrLoggingGlobalModule, self).setUp()

        self.mock_get_resource_connection = patch(
            "ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module_base."
            "get_resource_connection"
        )
        self.get_resource_connection = (
            self.mock_get_resource_connection.start()
        )

        self.mock_get_config = patch(
            "ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.logging_global.logging_global."
            "Logging_globalFacts.get_config"
        )
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestIosxrLoggingGlobalModule, self).tearDown()
        self.get_resource_connection.stop()
        self.get_config.stop()

    def test_iosxr_logging_global_merged_idempotent(self):
        run_cfg = dedent(
            """\
                logging tls-server test
                 vrf test
                 trustpoint test2
                 tls-hostname test2
                !
                logging file test path test maxfilesize 1024 severity info
                logging file test2 path test1 maxfilesize 1024 severity debugging
                logging ipv4 dscp af11
                logging ipv6 precedence routine
                logging trap informational
                logging events filter
                 match test
                 match test1
                !
                logging events threshold 10
                logging events buffer-size 1024
                logging events display-location
                logging events level warnings
                logging format rfc5424
                logging archive
                 device disk0
                 severity alerts
                 file-size 1
                 frequency daily
                 archive-size 1
                 archive-length 1
                !
                logging console warning
                logging console discriminator
                 match1 test
                 nomatch1 test3
                !
                logging history size 10
                logging monitor errors
                logging monitor discriminator
                 match1 test1
                !
                logging buffered 2097152
                logging buffered warnings
                logging buffered discriminator
                 match2 test
                !
                logging 1.1.1.1 vrf default severity critical port default
                logging correlator rule test type stateful
                 reissue-nonbistate
                 timeout 5
                 reparent
                 context-correlation
                !
                logging correlator rule test1 type nonstateful
                 timeout 6
                 context-correlation
                !
                logging correlator ruleset test
                 rulename test
                 rulename test1
                !
                logging correlator buffer-size 1024
                logging localfilesize 1024
                logging source-interface GigabitEthernet0/0/0/0 vrf test
                logging hostnameprefix test
                logging suppress duplicates
                !
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    archive=dict(
                        archive_length=1,
                        archive_size=1,
                        device="disk0",
                        file_size=1,
                        frequency="daily",
                        severity="alerts",
                    ),
                    buffered=dict(
                        size=2097152,
                        severity="warnings",
                        discriminator=[
                            dict(match_params="match2", name="test")
                        ],
                    ),
                    console=dict(
                        severity="warning",
                        discriminator=[
                            dict(match_params="match1", name="test"),
                            dict(match_params="nomatch1", name="test3"),
                        ],
                    ),
                    correlator=dict(
                        buffer_size=1024,
                        rule_sets=[
                            dict(name="test", rulename=["test1", "test"])
                        ],
                        rules=[
                            dict(
                                rule_name="test",
                                rule_type="stateful",
                                timeout=5,
                                context_correlation=True,
                                reissue_nonbistate=True,
                                reparent=True,
                            ),
                            dict(
                                rule_name="test1",
                                rule_type="nonstateful",
                                timeout=6,
                                context_correlation=True,
                            ),
                        ],
                    ),
                    events=dict(
                        severity="warnings",
                        display_location=True,
                        buffer_size=1024,
                        filter_match=["test1", "test"],
                        threshold=10,
                    ),
                    format=True,
                    files=[
                        dict(
                            maxfilesize=1024,
                            name="test",
                            path="test",
                            severity="info",
                        ),
                        dict(
                            maxfilesize=1024,
                            name="test2",
                            path="test1",
                            severity="debugging",
                        ),
                    ],
                    history=dict(size=10),
                    hostnameprefix="test",
                    hosts=[
                        dict(
                            host="1.1.1.1",
                            port="default",
                            severity="critical",
                            vrf="default",
                        )
                    ],
                    ipv6=dict(precedence="routine"),
                    ipv4=dict(dscp="af11"),
                    localfilesize=1024,
                    monitor=dict(
                        severity="errors",
                        discriminator=[
                            dict(match_params="match1", name="test1")
                        ],
                    ),
                    source_interfaces=[
                        dict(interface="GigabitEthernet0/0/0/0", vrf="test")
                    ],
                    suppress=dict(duplicates=True),
                    tls_servers=[
                        dict(
                            name="test",
                            tls_hostname="test2",
                            trustpoint="test2",
                            vrf="test",
                        )
                    ],
                    trap=dict(severity="informational"),
                ),
                state="merged",
            )
        )
        self.execute_module(changed=False, commands=[])

    def test_iosxr_logging_global_merged(self):
        set_module_args(
            dict(
                config=dict(
                    archive=dict(
                        archive_length=1,
                        archive_size=1,
                        device="disk0",
                        file_size=1,
                        frequency="daily",
                        severity="alerts",
                    ),
                    buffered=dict(
                        size=2097152,
                        severity="warnings",
                        discriminator=[
                            dict(match_params="match2", name="test")
                        ],
                    ),
                    console=dict(
                        severity="warning",
                        discriminator=[
                            dict(match_params="match1", name="test"),
                            dict(match_params="nomatch1", name="test3"),
                        ],
                    ),
                    correlator=dict(
                        buffer_size=1024,
                        rule_sets=[
                            dict(name="test", rulename=["test1", "test"])
                        ],
                        rules=[
                            dict(
                                rule_name="test",
                                rule_type="stateful",
                                timeout=5,
                                context_correlation=True,
                                reissue_nonbistate=True,
                                reparent=True,
                            ),
                            dict(
                                rule_name="test1",
                                rule_type="nonstateful",
                                timeout=6,
                                context_correlation=True,
                            ),
                        ],
                    ),
                    events=dict(
                        severity="warnings",
                        display_location=True,
                        buffer_size=1024,
                        filter_match=["test1", "test"],
                        threshold=10,
                    ),
                    format=True,
                    files=[
                        dict(
                            maxfilesize=1024,
                            name="test",
                            path="test",
                            severity="info",
                        ),
                        dict(
                            maxfilesize=1024,
                            name="test2",
                            path="test1",
                            severity="debugging",
                        ),
                    ],
                    history=dict(state="disabled", size=10),
                    hostnameprefix="test",
                    hosts=[
                        dict(
                            host="1.1.1.1",
                            port="default",
                            severity="critical",
                            vrf="default",
                        )
                    ],
                    ipv6=dict(precedence="routine"),
                    ipv4=dict(dscp="af11"),
                    localfilesize=1024,
                    monitor=dict(
                        severity="errors",
                        discriminator=[
                            dict(match_params="match1", name="test1")
                        ],
                    ),
                    source_interfaces=[
                        dict(interface="GigabitEthernet0/0/0/0", vrf="test")
                    ],
                    suppress=dict(duplicates=True),
                    tls_servers=[
                        dict(
                            name="test",
                            tls_hostname="test2",
                            trustpoint="test2",
                            vrf="test",
                        )
                    ],
                    trap=dict(severity="informational"),
                ),
                state="merged",
            )
        )
        commands = [
            "logging archive device disk0",
            "logging archive frequency daily",
            "logging archive severity alerts",
            "logging archive archive-size 1",
            "logging archive archive-length 1",
            "logging archive file-size 1",
            "logging buffered 2097152",
            "logging buffered warnings",
            "logging console warning",
            "logging correlator buffer-size 1024",
            "logging events threshold 10",
            "logging events buffer-size 1024",
            "logging events display-location",
            "logging events level warnings",
            "logging hostnameprefix test",
            "logging format rfc5424",
            "logging ipv4 dscp af11",
            "logging ipv6 precedence routine",
            "logging localfilesize 1024",
            "logging suppress duplicates",
            "logging monitor errors",
            "logging history size 10",
            "logging history disable",
            "logging trap informational",
            "logging 1.1.1.1 vrf default severity critical port default",
            "logging file test path test maxfilesize 1024 severity info",
            "logging file test2 path test1 maxfilesize 1024 severity debugging",
            "logging source-interface GigabitEthernet0/0/0/0 vrf test",
            "logging tls-server test tls-hostname test2",
            "logging tls-server test trustpoint test2",
            "logging tls-server test vrf test",
            "logging correlator ruleset  test  rulename test1",
            "logging correlator ruleset  test  rulename test",
            "logging correlator rule test type stateful timeout 5",
            "logging correlator rule test type stateful reissue-nonbistate",
            "logging correlator rule test type stateful reparent",
            "logging correlator rule test type stateful context-correlation",
            "logging correlator rule test1 type nonstateful timeout 6",
            "logging correlator rule test1 type nonstateful context-correlation",
            "logging events filter match test1",
            "logging events filter match test",
            "logging buffered discriminator match2 test",
            "logging monitor discriminator match1 test1",
            "logging console discriminator match1 test",
            "logging console discriminator nomatch1 test3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_logging_global_deleted(self):
        self.maxDiff = None
        run_cfg = dedent(
            """\
                logging tls-server test
                 vrf test
                 trustpoint test2
                 tls-hostname test2
                !
                logging file test path test maxfilesize 1024 severity info
                logging file test2 path test1 maxfilesize 1024 severity debugging
                logging ipv4 dscp af11
                logging ipv6 precedence routine
                logging trap informational
                logging events filter
                 match test
                 match test1
                !
                logging events threshold 10
                logging events buffer-size 1024
                logging events display-location
                logging events level warnings
                logging format rfc5424
                logging archive
                 device disk0
                 severity alerts
                 file-size 1
                 frequency daily
                 archive-size 1
                 archive-length 1
                !
                logging console warning
                logging console discriminator
                 match1 test
                 nomatch1 test3
                !
                logging history size 10
                logging history disable
                logging monitor errors
                logging monitor discriminator
                 match1 test1
                !
                logging buffered 2097152
                logging buffered warnings
                logging buffered discriminator
                 match2 test
                !
                logging 1.1.1.1 vrf default severity critical port default
                logging correlator rule test type stateful
                 reissue-nonbistate
                 timeout 5
                 reparent
                 context-correlation
                !
                logging correlator rule test1 type nonstateful
                 timeout 6
                 context-correlation
                !
                logging correlator ruleset test
                 rulename test
                 rulename test1
                !
                logging correlator buffer-size 1024
                logging localfilesize 1024
                logging source-interface GigabitEthernet0/0/0/0 vrf test
                logging hostnameprefix test
                logging suppress duplicates
                !
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(dict(state="deleted"))
        commands = [
            "no logging archive device disk0",
            "no logging archive frequency daily",
            "no logging archive severity alerts",
            "no logging archive archive-size 1",
            "no logging archive archive-length 1",
            "no logging archive file-size 1",
            "no logging buffered 2097152",
            "no logging buffered warnings",
            "no logging console warning",
            "no logging correlator buffer-size 1024",
            "no logging events threshold 10",
            "no logging events buffer-size 1024",
            "no logging events display-location",
            "no logging events level warnings",
            "no logging hostnameprefix test",
            "no logging format rfc5424",
            "no logging ipv4 dscp af11",
            "no logging ipv6 precedence routine",
            "no logging localfilesize 1024",
            "no logging suppress duplicates",
            "no logging monitor errors",
            "no logging history size 10",
            "no logging history disable",
            "no logging trap informational",
            "no logging 1.1.1.1 vrf default severity critical port default",
            "no logging file test path test maxfilesize 1024 severity info",
            "no logging file test2 path test1 maxfilesize 1024 severity debugging",
            "no logging source-interface GigabitEthernet0/0/0/0 vrf test",
            "no logging tls-server test",
            "no logging correlator ruleset  test  rulename test",
            "no logging correlator ruleset  test  rulename test1",
            "no logging correlator rule test type stateful timeout 5",
            "no logging correlator rule test type stateful reissue-nonbistate",
            "no logging correlator rule test type stateful reparent",
            "no logging correlator rule test type stateful context-correlation",
            "no logging correlator rule test1 type nonstateful timeout 6",
            "no logging correlator rule test1 type nonstateful context-correlation",
            "no logging events filter match test",
            "no logging events filter match test1",
            "no logging buffered discriminator match2 test",
            "no logging monitor discriminator match1 test1",
            "no logging console discriminator match1 test",
            "no logging console discriminator nomatch1 test3",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_logging_global_replaced(self):
        self.maxDiff = None
        run_cfg = dedent(
            """\
                logging tls-server test
                 vrf test
                 trustpoint test2
                 tls-hostname test2
                !
                logging file test path test maxfilesize 1024 severity info
                logging file test2 path test1 maxfilesize 1024 severity debugging
                logging ipv4 dscp af11
                logging ipv6 precedence routine
                logging trap informational
                logging events filter
                 match test
                 match test1
                !
                logging events threshold 10
                logging events buffer-size 1024
                logging events display-location
                logging events level warnings
                logging format rfc5424
                logging archive
                 device disk0
                 severity alerts
                 file-size 1
                 frequency daily
                 archive-size 1
                 archive-length 1
                !
                logging console warning
                logging console discriminator
                 match1 test
                 nomatch1 test3
                !
                logging history size 10
                logging history disable
                logging monitor errors
                logging monitor discriminator
                 match1 test1
                !
                logging buffered 2097152
                logging buffered warnings
                logging buffered discriminator
                 match2 test
                !
                logging 1.1.1.1 vrf default severity critical port default
                logging correlator rule test type stateful
                 reissue-nonbistate
                 timeout 5
                 reparent
                 context-correlation
                !
                logging correlator rule test1 type nonstateful
                 timeout 6
                 context-correlation
                !
                logging correlator ruleset test
                 rulename test
                 rulename test1
                !
                logging correlator buffer-size 1024
                logging localfilesize 1024
                logging source-interface GigabitEthernet0/0/0/0 vrf test
                logging hostnameprefix test
                logging suppress duplicates
                !
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    archive=dict(
                        archive_length=1,
                        archive_size=1,
                        device="disk0",
                        file_size=1,
                        severity="alerts",
                    ),
                    buffered=dict(size=2097152, severity="warnings"),
                    console=dict(
                        severity="warning",
                        discriminator=[
                            dict(match_params="match1", name="test1")
                        ],
                    ),
                    correlator=dict(
                        buffer_size=1024,
                        rules=[
                            dict(
                                rule_name="test",
                                rule_type="stateful",
                                timeout=6,
                                context_correlation=True,
                                reissue_nonbistate=True,
                                reparent=True,
                            )
                        ],
                    ),
                    events=dict(
                        severity="warnings",
                        display_location=True,
                        buffer_size=1024,
                        filter_match=["test1"],
                        threshold=12,
                    ),
                    format=True,
                    files=[
                        dict(
                            maxfilesize=1024,
                            name="test",
                            path="test1",
                            severity="info",
                        )
                    ],
                    history=dict(state="disabled", size=10),
                    hostnameprefix="test",
                    hosts=[
                        dict(
                            host="1.1.1.2",
                            port="default",
                            severity="critical",
                            vrf="default",
                        )
                    ],
                    ipv6=dict(precedence="routine"),
                    ipv4=dict(dscp="af11"),
                    localfilesize=1024,
                    source_interfaces=[
                        dict(interface="GigabitEthernet0/0/0/0", vrf="test")
                    ],
                    tls_servers=[
                        dict(
                            name="test",
                            tls_hostname="test2",
                            trustpoint="test3",
                            vrf="test",
                        )
                    ],
                ),
                state="replaced",
            )
        )
        commands = [
            "logging console discriminator match1 test1",
            "logging correlator rule test type stateful timeout 6",
            "logging events threshold 12",
            "logging 1.1.1.2 vrf default severity critical port default",
            "logging file test path test1 maxfilesize 1024 severity info",
            "logging tls-server test trustpoint test3",
            "no logging 1.1.1.1 vrf default severity critical port default",
            "no logging archive frequency daily",
            "no logging buffered discriminator match2 test",
            "no logging console discriminator match1 test",
            "no logging console discriminator nomatch1 test3",
            "no logging correlator rule test1 type nonstateful context-correlation",
            "no logging correlator rule test1 type nonstateful timeout 6",
            "no logging correlator ruleset  test  rulename test",
            "no logging correlator ruleset  test  rulename test1",
            "no logging events filter match test",
            "no logging file test2 path test1 maxfilesize 1024 severity debugging",
            "no logging monitor discriminator match1 test1",
            "no logging monitor errors",
            "no logging suppress duplicates",
            "no logging trap informational",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_logging_global_rendered(self):
        self.maxDiff = None
        set_module_args(
            dict(
                config=dict(
                    archive=dict(
                        archive_length=1,
                        archive_size=1,
                        device="disk0",
                        file_size=1,
                        frequency="daily",
                        severity="alerts",
                    ),
                    buffered=dict(
                        size=2097152,
                        severity="warnings",
                        discriminator=[
                            dict(match_params="match2", name="test")
                        ],
                    ),
                    console=dict(
                        severity="warning",
                        discriminator=[
                            dict(match_params="match1", name="test"),
                            dict(match_params="nomatch1", name="test3"),
                        ],
                    ),
                    correlator=dict(
                        buffer_size=1024,
                        rule_sets=[
                            dict(name="test", rulename=["test1", "test"])
                        ],
                        rules=[
                            dict(
                                rule_name="test",
                                rule_type="stateful",
                                timeout=5,
                                context_correlation=True,
                                reissue_nonbistate=True,
                                reparent=True,
                            ),
                            dict(
                                rule_name="test1",
                                rule_type="nonstateful",
                                timeout=6,
                                context_correlation=True,
                            ),
                        ],
                    ),
                    events=dict(
                        severity="warnings",
                        display_location=True,
                        buffer_size=1024,
                        filter_match=["test1", "test"],
                        threshold=10,
                    ),
                    format=True,
                    files=[
                        dict(
                            maxfilesize=1024,
                            name="test",
                            path="test",
                            severity="info",
                        ),
                        dict(
                            maxfilesize=1024,
                            name="test2",
                            path="test1",
                            severity="debugging",
                        ),
                    ],
                    history=dict(state="disabled", size=10),
                    hostnameprefix="test",
                    hosts=[
                        dict(
                            host="1.1.1.1",
                            port="default",
                            severity="critical",
                            vrf="default",
                        )
                    ],
                    ipv6=dict(precedence="routine"),
                    ipv4=dict(dscp="af11"),
                    localfilesize=1024,
                    monitor=dict(
                        severity="errors",
                        discriminator=[
                            dict(match_params="match1", name="test1")
                        ],
                    ),
                    source_interfaces=[
                        dict(interface="GigabitEthernet0/0/0/0", vrf="test")
                    ],
                    suppress=dict(duplicates=True),
                    tls_servers=[
                        dict(
                            name="test",
                            tls_hostname="test2",
                            trustpoint="test2",
                            vrf="test",
                        )
                    ],
                    trap=dict(severity="informational"),
                ),
                state="rendered",
            )
        )
        commands = [
            "logging archive device disk0",
            "logging archive frequency daily",
            "logging archive severity alerts",
            "logging archive archive-size 1",
            "logging archive archive-length 1",
            "logging archive file-size 1",
            "logging buffered 2097152",
            "logging buffered warnings",
            "logging console warning",
            "logging correlator buffer-size 1024",
            "logging events threshold 10",
            "logging events buffer-size 1024",
            "logging events display-location",
            "logging events level warnings",
            "logging hostnameprefix test",
            "logging format rfc5424",
            "logging ipv4 dscp af11",
            "logging ipv6 precedence routine",
            "logging localfilesize 1024",
            "logging suppress duplicates",
            "logging monitor errors",
            "logging history size 10",
            "logging history disable",
            "logging trap informational",
            "logging 1.1.1.1 vrf default severity critical port default",
            "logging file test path test maxfilesize 1024 severity info",
            "logging file test2 path test1 maxfilesize 1024 severity debugging",
            "logging source-interface GigabitEthernet0/0/0/0 vrf test",
            "logging tls-server test tls-hostname test2",
            "logging tls-server test trustpoint test2",
            "logging tls-server test vrf test",
            "logging correlator ruleset  test  rulename test1",
            "logging correlator ruleset  test  rulename test",
            "logging correlator rule test type stateful timeout 5",
            "logging correlator rule test type stateful reissue-nonbistate",
            "logging correlator rule test type stateful reparent",
            "logging correlator rule test type stateful context-correlation",
            "logging correlator rule test1 type nonstateful timeout 6",
            "logging correlator rule test1 type nonstateful context-correlation",
            "logging events filter match test1",
            "logging events filter match test",
            "logging buffered discriminator match2 test",
            "logging monitor discriminator match1 test1",
            "logging console discriminator match1 test",
            "logging console discriminator nomatch1 test3",
        ]
        result = self.execute_module(changed=False)
        self.assertEqual(sorted(result["rendered"]), sorted(commands))

    def test_iosxr_logging_global_overridden(self):
        self.maxDiff = None
        run_cfg = dedent(
            """\
                logging tls-server test
                 vrf test
                 trustpoint test2
                 tls-hostname test2
                !
                logging file test path test maxfilesize 1024 severity info
                logging file test2 path test1 maxfilesize 1024 severity debugging
                logging ipv4 dscp af11
                logging ipv6 precedence routine
                logging trap informational
                logging events filter
                 match test
                 match test1
                !
                logging events threshold 10
                logging events buffer-size 1024
                logging events display-location
                logging events level warnings
                logging format rfc5424
                logging archive
                 device disk0
                 severity alerts
                 file-size 1
                 frequency daily
                 archive-size 1
                 archive-length 1
                !
                logging console warning
                logging console discriminator
                 match1 test
                 nomatch1 test3
                !
                logging history size 10
                logging history disable
                logging monitor errors
                logging monitor discriminator
                 match1 test1
                !
                logging buffered 2097152
                logging buffered warnings
                logging buffered discriminator
                 match2 test
                !
                logging 1.1.1.1 vrf default severity critical port default
                logging correlator rule test type stateful
                 reissue-nonbistate
                 timeout 5
                 reparent
                 context-correlation
                !
                logging correlator rule test1 type nonstateful
                 timeout 6
                 context-correlation
                !
                logging correlator ruleset test
                 rulename test
                 rulename test1
                !
                logging correlator buffer-size 1024
                logging localfilesize 1024
                logging source-interface GigabitEthernet0/0/0/0 vrf test
                logging hostnameprefix test
                logging suppress duplicates
                !
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(
            dict(
                config=dict(
                    archive=dict(
                        archive_length=1,
                        archive_size=1,
                        device="disk0",
                        file_size=1,
                        severity="alerts",
                    ),
                    buffered=dict(size=2097152, severity="warnings"),
                    console=dict(
                        severity="warning",
                        discriminator=[
                            dict(match_params="match1", name="test1")
                        ],
                    ),
                    correlator=dict(
                        buffer_size=1024,
                        rules=[
                            dict(
                                rule_name="test",
                                rule_type="stateful",
                                timeout=6,
                                context_correlation=True,
                                reissue_nonbistate=True,
                                reparent=True,
                            )
                        ],
                    ),
                    events=dict(
                        severity="warnings",
                        display_location=True,
                        buffer_size=1024,
                        filter_match=["test1"],
                        threshold=12,
                    ),
                    format=True,
                    files=[
                        dict(
                            maxfilesize=1024,
                            name="test",
                            path="test1",
                            severity="info",
                        )
                    ],
                    history=dict(state="disabled", size=10),
                    hostnameprefix="test",
                    hosts=[
                        dict(
                            host="1.1.1.2",
                            port="default",
                            severity="critical",
                            vrf="default",
                        )
                    ],
                    ipv6=dict(precedence="routine"),
                    ipv4=dict(dscp="af11"),
                    localfilesize=1024,
                    source_interfaces=[
                        dict(interface="GigabitEthernet0/0/0/0", vrf="test")
                    ],
                    tls_servers=[
                        dict(
                            name="test",
                            tls_hostname="test2",
                            trustpoint="test3",
                            vrf="test",
                        )
                    ],
                ),
                state="overridden",
            )
        )
        commands = [
            "logging console discriminator match1 test1",
            "logging correlator rule test type stateful timeout 6",
            "logging events threshold 12",
            "logging 1.1.1.2 vrf default severity critical port default",
            "logging file test path test1 maxfilesize 1024 severity info",
            "logging tls-server test trustpoint test3",
            "no logging 1.1.1.1 vrf default severity critical port default",
            "no logging archive frequency daily",
            "no logging buffered discriminator match2 test",
            "no logging console discriminator match1 test",
            "no logging console discriminator nomatch1 test3",
            "no logging correlator rule test1 type nonstateful context-correlation",
            "no logging correlator rule test1 type nonstateful timeout 6",
            "no logging correlator ruleset  test  rulename test",
            "no logging correlator ruleset  test  rulename test1",
            "no logging events filter match test",
            "no logging file test2 path test1 maxfilesize 1024 severity debugging",
            "no logging monitor discriminator match1 test1",
            "no logging monitor errors",
            "no logging suppress duplicates",
            "no logging trap informational",
        ]
        result = self.execute_module(changed=True)
        self.assertEqual(sorted(result["commands"]), sorted(commands))

    def test_iosxr_logging_global_gathered(self):
        run_cfg = dedent(
            """\
                logging tls-server test
                 vrf test
                 trustpoint test2
                 tls-hostname test2
                !
                logging file test path test maxfilesize 1024 severity info
                logging ipv4 dscp af11
                logging ipv6 precedence routine
                logging trap informational
                logging events filter
                 match test
                !
                logging events threshold 10
                logging events buffer-size 1024
                logging events display-location
                logging events level warnings
                logging format rfc5424
                logging archive
                 device disk0
                 severity alerts
                 file-size 1
                 frequency daily
                 archive-size 1
                 archive-length 1
                !
                logging console warning
                logging console discriminator
                 match1 test
                !
                logging history size 10
                logging history disable
                logging monitor errors
                logging monitor discriminator
                 match1 test1
                !
                logging buffered 2097152
                logging buffered warnings
                logging buffered discriminator
                 match2 test
                !
                logging 1.1.1.1 vrf default severity critical port default
                logging correlator rule test type stateful
                 reissue-nonbistate
                 timeout 5
                 reparent
                 context-correlation
                !
                logging correlator ruleset test
                 rulename test
                !
                logging correlator buffer-size 1024
                logging localfilesize 1024
                logging source-interface GigabitEthernet0/0/0/0 vrf test
                logging hostnameprefix test
                logging suppress duplicates
                !
            """
        )
        self.get_config.return_value = run_cfg
        set_module_args(dict(state="gathered"))
        gathered = {
            "archive": {
                "archive_length": 1,
                "archive_size": 1,
                "device": "disk0",
                "file_size": 1,
                "frequency": "daily",
                "severity": "alerts",
            },
            "buffered": {
                "discriminator": [{"match_params": "match2", "name": "test"}],
                "severity": "warnings",
                "size": 2097152,
            },
            "console": {
                "discriminator": [{"match_params": "match1", "name": "test"}],
                "severity": "warning",
            },
            "correlator": {
                "buffer_size": 1024,
                "rule_sets": [{"name": "test", "rulename": ["test"]}],
                "rules": [
                    {
                        "context_correlation": True,
                        "reissue_nonbistate": True,
                        "reparent": True,
                        "rule_name": "test",
                        "rule_type": "stateful",
                        "timeout": 5,
                    }
                ],
            },
            "events": {
                "buffer_size": 1024,
                "display_location": True,
                "filter_match": ["test"],
                "severity": "warnings",
                "threshold": 10,
            },
            "files": [
                {
                    "maxfilesize": 1024,
                    "name": "test",
                    "path": "test",
                    "severity": "info",
                }
            ],
            "format": True,
            "history": {"state": "disabled", "size": 10},
            "hostnameprefix": "test",
            "hosts": [
                {
                    "host": "1.1.1.1",
                    "port": "default",
                    "severity": "critical",
                    "vrf": "default",
                }
            ],
            "ipv4": {"dscp": "af11"},
            "ipv6": {"precedence": "routine"},
            "localfilesize": 1024,
            "monitor": {
                "discriminator": [{"match_params": "match1", "name": "test1"}],
                "severity": "errors",
            },
            "source_interfaces": [
                {"interface": "GigabitEthernet0/0/0/0", "vrf": "test"}
            ],
            "suppress": {"duplicates": True},
            "tls_servers": [
                {
                    "name": "test",
                    "tls_hostname": "test2",
                    "trustpoint": "test2",
                    "vrf": "test",
                }
            ],
            "trap": {"severity": "informational"},
        }
        result = self.execute_module(changed=False)
        self.assertEqual(gathered, result["gathered"])

    def test_iosxr_logging_global_parsed(self):
        set_module_args(
            dict(
                running_config="logging tls-server test\n vrf test\n trustpoint test2\n tls-hostname test2"
                "\n!\nlogging file test path test maxfilesize 1024 severity info\nlogging ipv4 dscp"
                " af11\nlogging ipv6 precedence routine\nlogging trap informational\nlogging events"
                " filter\n match test1\n!\nlogging events threshold "
                "10\nlogging events buffer-size 1024\nlogging events display-location"
                "\nlogging events level warnings"
                "\nlogging format rfc5424\nlogging archive\n device disk0"
                "\n severity alerts\n file-size 1\n frequency "
                "daily\n archive-size 1\n archive-length 1\n!\nlogging console "
                "warning\nlogging console discriminator\n "
                "match1 test\n!\nlogging history size "
                "10\nlogging history disable\nlogging monitor errors"
                "\nlogging monitor discriminator\n match1 test1\n!"
                "\nlogging buffered 2097152\nlogging buffered warnings\n"
                "logging buffered discriminator\n match2 test\n!\nlogging "
                "1.1.1.1 vrf default severity critical port default"
                "\nlogging correlator rule test type stateful\n reissue-nonbistate\n "
                "timeout 5\n reparent\n context-correlation\n!"
                "\n!\nlogging correlator ruleset test\n rulename test1"
                "\n!\nlogging correlator buffer-size 1024\nlogging "
                "localfilesize 1024\nlogging source-interface"
                " GigabitEthernet0/0/0/0 vrf test\nlogging hostnameprefix "
                "test\nlogging suppress duplicates",
                state="parsed",
            )
        )
        result = self.execute_module(changed=False)
        parsed_list = {
            "archive": {
                "archive_length": 1,
                "archive_size": 1,
                "device": "disk0",
                "file_size": 1,
                "frequency": "daily",
                "severity": "alerts",
            },
            "buffered": {
                "discriminator": [{"match_params": "match2", "name": "test"}],
                "severity": "warnings",
                "size": 2097152,
            },
            "console": {
                "discriminator": [{"match_params": "match1", "name": "test"}],
                "severity": "warning",
            },
            "correlator": {
                "buffer_size": 1024,
                "rule_sets": [{"name": "test", "rulename": ["test1"]}],
                "rules": [
                    {
                        "context_correlation": True,
                        "reissue_nonbistate": True,
                        "reparent": True,
                        "rule_name": "test",
                        "rule_type": "stateful",
                        "timeout": 5,
                    }
                ],
            },
            "events": {
                "buffer_size": 1024,
                "display_location": True,
                "filter_match": ["test1"],
                "severity": "warnings",
                "threshold": 10,
            },
            "files": [
                {
                    "maxfilesize": 1024,
                    "name": "test",
                    "path": "test",
                    "severity": "info",
                }
            ],
            "format": True,
            "history": {"state": "disabled", "size": 10},
            "hostnameprefix": "test",
            "hosts": [
                {
                    "host": "1.1.1.1",
                    "port": "default",
                    "severity": "critical",
                    "vrf": "default",
                }
            ],
            "ipv4": {"dscp": "af11"},
            "ipv6": {"precedence": "routine"},
            "localfilesize": 1024,
            "monitor": {
                "discriminator": [{"match_params": "match1", "name": "test1"}],
                "severity": "errors",
            },
            "source_interfaces": [
                {"interface": "GigabitEthernet0/0/0/0", "vrf": "test"}
            ],
            "suppress": {"duplicates": True},
            "tls_servers": [
                {
                    "name": "test",
                    "tls_hostname": "test2",
                    "trustpoint": "test2",
                    "vrf": "test",
                }
            ],
            "trap": {"severity": "informational"},
        }

        self.assertEqual(parsed_list, result["parsed"])



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: node_group_node_info
short_description: Information module for Node Group Node
description:
- Get all Node Group Node.
- This API retrieves the list of nodes associated with a node group in the cluster with a given node group name.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  nodeGroupName:
    description:
    - NodeGroupName path parameter. Name of the existing node group.
    type: str
requirements:
- ciscoisesdk >= 2.0.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Node Group
  description: Complete reference of the Node Group API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deployment-openapi
notes:
  - SDK Method used are
    node_group.NodeGroup.get_nodes,

  - Paths used are
    get /api/v1/deployment/node-group/{nodeGroupName}/node,

"""

EXAMPLES = r"""
- name: Get all Node Group Node
  cisco.ise.node_group_node_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    nodeGroupName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "hostname": "string"
      }
    ]
"""



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: ec2_vpc_dhcp_option_info
version_added: 1.0.0
short_description: Gather information about dhcp options sets in AWS
description:
    - Gather information about dhcp options sets in AWS.
author: "Nick Aslanidis (@naslanidis)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeDhcpOptions.html) for possible filters.
    type: dict
  dhcp_options_ids:
    description:
      - Get details of specific DHCP Option IDs.
    aliases: ['DhcpOptionIds']
    type: list
    elements: str
  dry_run:
    description:
      - Checks whether you have the required permissions to view the DHCP
        Options.
    aliases: ['DryRun']
    type: bool
    default: false
extends_documentation_fragment:
- amazon.aws.aws
- amazon.aws.ec2

'''

EXAMPLES = '''
# # Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all DHCP Option sets for an account or profile
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
  register: dhcp_info

- name: Gather information about a filtered list of DHCP Option sets
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
    filters:
        "tag:Name": "abc-123"
  register: dhcp_info

- name: Gather information about a specific DHCP Option set by DhcpOptionId
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
    DhcpOptionsIds: dopt-123fece2
  register: dhcp_info

'''

RETURN = '''
dhcp_options:
    description: The DHCP options created, associated or found
    returned: always
    type: list
    elements: dict
    contains:
        dhcp_configurations:
            description: The DHCP configuration for the option set
            type: list
            sample:
              - '{"key": "ntp-servers", "values": [{"value": "10.0.0.2" , "value": "10.0.1.2"}]}'
              - '{"key": "netbios-name-servers", "values": [{value": "10.0.0.1"}, {"value": "10.0.1.1" }]}'
        dhcp_options_id:
            description: The aws resource id of the primary DCHP options set created or found
            type: str
            sample: "dopt-0955331de6a20dd07"
        owner_id:
            description: The ID of the AWS account that owns the DHCP options set.
            type: str
            sample: 012345678912
        tags:
            description: The tags to be applied to a DHCP options set
            type: list
            sample:
              - '{"Key": "CreatedBy", "Value": "ansible-test"}'
              - '{"Key": "Collection", "Value": "amazon.aws"}'
dhcp_config:
    description: The boto2-style DHCP options created, associated or found. Provided for consistency with ec2_vpc_dhcp_option's C(dhcp_config).
    returned: always
    type: list
    contains:
      domain-name-servers:
        description: The IP addresses of up to four domain name servers, or AmazonProvidedDNS.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      domain-name:
        description: The domain name for hosts in the DHCP option sets
        returned: when available
        type: list
        sample:
          - "my.example.com"
      ntp-servers:
        description: The IP addresses of up to four Network Time Protocol (NTP) servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-name-servers:
        description: The IP addresses of up to four NetBIOS name servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-node-type:
        description: The NetBIOS node type (1, 2, 4, or 8).
        returned: when available
        type: str
        sample: 2
changed:
    description: True if listing the dhcp options succeeds
    type: bool
    returned: always
'''

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ..module_utils.core import AnsibleAWSModule
from ..module_utils.ec2 import AWSRetry
from ..module_utils.ec2 import ansible_dict_to_boto3_filter_list
from ..module_utils.ec2 import boto3_tag_list_to_ansible_dict
from ..module_utils.ec2 import normalize_ec2_vpc_dhcp_config


def get_dhcp_options_info(dhcp_option):
    dhcp_option_info = {'DhcpOptionsId': dhcp_option['DhcpOptionsId'],
                        'DhcpConfigurations': dhcp_option['DhcpConfigurations'],
                        'Tags': boto3_tag_list_to_ansible_dict(dhcp_option.get('Tags', [{'Value': '', 'Key': 'Name'}]))}
    return dhcp_option_info


def list_dhcp_options(client, module):
    params = dict(Filters=ansible_dict_to_boto3_filter_list(module.params.get('filters')))

    if module.params.get("dry_run"):
        params['DryRun'] = True

    if module.params.get("dhcp_options_ids"):
        params['DhcpOptionsIds'] = module.params.get("dhcp_options_ids")

    try:
        all_dhcp_options = client.describe_dhcp_options(aws_retry=True, **params)
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    normalized_config = [normalize_ec2_vpc_dhcp_config(config['DhcpConfigurations']) for config in all_dhcp_options['DhcpOptions']]
    raw_config = [camel_dict_to_snake_dict(get_dhcp_options_info(option), ignore_list=['Tags']) for option in all_dhcp_options['DhcpOptions']]
    return raw_config, normalized_config


def main():
    argument_spec = dict(
        filters=dict(type='dict', default={}),
        dry_run=dict(type='bool', default=False, aliases=['DryRun']),
        dhcp_options_ids=dict(type='list', elements='str', aliases=['DhcpOptionIds'])
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True
    )

    client = module.client('ec2', retry_decorator=AWSRetry.jittered_backoff())

    # call your function here
    results, normalized_config = list_dhcp_options(client, module)

    module.exit_json(dhcp_options=results, dhcp_config=normalized_config)


if __name__ == '__main__':
    main()



#
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import sys
import copy

from ansible import constants as C
from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    eos_provider_spec,
)
from ansible_collections.ansible.netcommon.plugins.action.network import (
    ActionModule as ActionNetworkModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    load_provider,
)
from ansible.utils.display import Display

display = Display()


class ActionModule(ActionNetworkModule):
    def run(self, tmp=None, task_vars=None):
        del tmp  # tmp no longer has any effect

        module_name = self._task.action.split(".")[-1]
        self._config_module = (
            True if module_name in ["eos_config", "config"] else False
        )
        persistent_connection = self._play_context.connection.split(".")[-1]
        warnings = []

        if persistent_connection in ("network_cli", "httpapi"):
            provider = self._task.args.get("provider", {})
            if any(provider.values()):
                display.warning(
                    "provider is unnecessary when using %s and will be ignored"
                    % self._play_context.connection
                )
                del self._task.args["provider"]
            if self._task.args.get("transport"):
                display.warning(
                    "transport is unnecessary when using %s and will be ignored"
                    % self._play_context.connection
                )
                del self._task.args["transport"]
        elif self._play_context.connection == "local":
            provider = load_provider(eos_provider_spec, self._task.args)
            transport = provider["transport"] or "cli"

            display.vvvv(
                "connection transport is %s" % transport,
                self._play_context.remote_addr,
            )

            if transport == "cli":
                pc = copy.deepcopy(self._play_context)
                pc.connection = "ansible.netcommon.network_cli"
                pc.network_os = "arista.eos.eos"
                pc.remote_addr = (
                    provider["host"] or self._play_context.remote_addr
                )
                pc.port = int(
                    provider["port"] or self._play_context.port or 22
                )
                pc.remote_user = (
                    provider["username"] or self._play_context.connection_user
                )
                pc.password = (
                    provider["password"] or self._play_context.password
                )
                pc.private_key_file = (
                    provider["ssh_keyfile"]
                    or self._play_context.private_key_file
                )
                pc.become = provider["authorize"] or False
                if pc.become:
                    pc.become_method = "enable"
                pc.become_pass = provider["auth_pass"]

                connection = self._shared_loader_obj.connection_loader.get(
                    "ansible.netcommon.persistent",
                    pc,
                    sys.stdin,
                    task_uuid=self._task._uuid,
                )

                # TODO: Remove below code after ansible minimal is cut out
                if connection is None:
                    pc.connection = "network_cli"
                    pc.network_os = "eos"
                    connection = self._shared_loader_obj.connection_loader.get(
                        "persistent", pc, sys.stdin, task_uuid=self._task._uuid
                    )

                display.vvv(
                    "using connection plugin %s (was local)" % pc.connection,
                    pc.remote_addr,
                )

                command_timeout = (
                    int(provider["timeout"])
                    if provider["timeout"]
                    else connection.get_option("persistent_command_timeout")
                )
                connection.set_options(
                    direct={"persistent_command_timeout": command_timeout}
                )

                socket_path = connection.run()
                display.vvvv("socket_path: %s" % socket_path, pc.remote_addr)
                if not socket_path:
                    return {
                        "failed": True,
                        "msg": "unable to open shell. Please see: "
                        + "https://docs.ansible.com/ansible/network_debug_troubleshooting.html#unable-to-open-shell",
                    }

                task_vars["ansible_socket"] = socket_path
                warnings.append(
                    [
                        "connection local support for this module is deprecated and will be removed in version 2.14,"
                        " use connection %s" % pc.connection
                    ]
                )
            else:
                self._task.args["provider"] = ActionModule.eapi_implementation(
                    provider, self._play_context
                )
                warnings.append(
                    [
                        "connection local support for this module is deprecated and will be removed in version 2.14,"
                        " use connection either httpapi or ansible.netcommon.httpapi (whichever is applicable)"
                    ]
                )
        else:
            return {
                "failed": True,
                "msg": "Connection type %s is not valid for this module"
                % self._play_context.connection,
            }

        result = super(ActionModule, self).run(task_vars=task_vars)
        if warnings:
            if "warnings" in result:
                result["warnings"].extend(warnings)
            else:
                result["warnings"] = warnings
        return result

    @staticmethod
    def eapi_implementation(provider, play_context):
        provider["transport"] = "eapi"

        if provider.get("host") is None:
            provider["host"] = play_context.remote_addr

        if provider.get("port") is None:
            default_port = 443 if provider["use_ssl"] else 80
            provider["port"] = int(play_context.port or default_port)

        if provider.get("timeout") is None:
            provider["timeout"] = C.PERSISTENT_COMMAND_TIMEOUT

        if provider.get("username") is None:
            provider["username"] = play_context.connection_user

        if provider.get("password") is None:
            provider["password"] = play_context.password

        if provider.get("authorize") is None:
            provider["authorize"] = False

        return provider
